package com.sisulizer.converter;

public class Common
{
  // Quantity
  public static final int LENGTH = 0;
  public static final int TEMPERATURE = 1;

  // Metric length
  public static final int KM = 0;
  public static final int M = 1;
  public static final int CM = 2;
  public static final int MM = 3;

  // US length
  public static final int MILE = 0;
  public static final int YARD = 1;
  public static final int FOOT = 2;
  public static final int INCH = 3;

  // Metric temperature
  public static final int CELSIUS = 0;
  public static final int KELVIN = 1;

  // US temperature
  public static final int FAHRENHEIT = 0;

  public static final double MILE_AS_METERS = 1609.3;
  public static final double YARD_AS_METERS = 0.9144;

  public static final int INCH_IN_FEET = 12;
  public static final int FEET_IN_YARD = 3;
  public static final int YARDS_IN_MILE = 1760;

  public static final double CELSIUS_OFFSET = 273.15;
  public static final double FAHRENHEIT_OFFSET = 32;
  public static final double FAHRENHEIT_FACTOR = 1.8;

  static public double metricLengthToUsLength(
    double value,
    int metricLength,
    int usLength)
  {
    double meters, yards;

    switch (metricLength)
    {
      case KM:
        meters = 1000*value;
        break;

      case CM:
        meters = value/100;
        break;

      case MM:
        meters = value/1000;
        break;

      default:
        meters = value;
        break;
    }

    yards = meters/YARD_AS_METERS;

    switch (usLength)
    {
      case MILE:
        return yards/YARDS_IN_MILE;

      case FOOT:
        return FEET_IN_YARD*yards;

      case INCH:
        return FEET_IN_YARD*INCH_IN_FEET*yards;

      default:
        return yards;
    }
  }

  static public double usLengthToMetricLength(
    double value,
    int usLength,
    int metricLength)
  {
    double meters, yards;

    switch (usLength)
    {
      case MILE:
        yards = YARDS_IN_MILE*value;
        break;

      case FOOT:
        yards = value/FEET_IN_YARD;
        break;

      case INCH:
        yards = value/(FEET_IN_YARD * INCH_IN_FEET);
        break;

      default:
        yards = value;
        break;
    }

    meters = YARD_AS_METERS*yards;

    switch (metricLength)
    {
      case KM:
        return meters/1000;

      case CM:
        return 100*meters;

      case MM:
        return 1000*meters;

      default:
        return meters;
    }
  }

  static public double metricTemperatureToUsTemperature(
    double value,
    int metricTemperature,
    int usTemperature)
  {
    double celsius;

    switch (metricTemperature)
    {
      case KELVIN:
        celsius = value - CELSIUS_OFFSET;
        break;

      default:
        celsius = value;
        break;
    }

    return FAHRENHEIT_FACTOR*celsius + FAHRENHEIT_OFFSET;
  }

  static public double usTemperatureToMetricTemperature(
    double value,
    int usTemperature,
    int metricTemperature)
  {
    double celsius;

    celsius = (value - FAHRENHEIT_OFFSET)/FAHRENHEIT_FACTOR;

    switch (metricTemperature)
    {
      case KELVIN:
        return celsius + CELSIUS_OFFSET;

      default:
        return celsius;
    }
  }
}