package com.sisulizer.converter;

import java.text.*;
import android.app.Activity;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.*;
import android.view.View.*;
import android.widget.*;
import android.widget.AdapterView.*;

public class ConverterActivity extends Activity implements OnClickListener, OnItemSelectedListener, OnKeyListener 
{
  private boolean updating = false;
  private Resources res;
  private Spinner typeSpinner;
  private Spinner metricSpinner;
  private EditText metricEdit;
  private Spinner usSpinner;
  private EditText usEdit;
  private Button button;

  private int getQuantity()
  {
    return typeSpinner.getSelectedItemPosition();
  }

  private int getMetricLength()
  {
    return metricSpinner.getSelectedItemPosition();
  }

  private int getUsLength()
  {
    return usSpinner.getSelectedItemPosition();
  }

  private int getMetricTemperature()
  {
    return metricSpinner.getSelectedItemPosition();
  }

  private int getUsTemperature()
  {
    return usSpinner.getSelectedItemPosition();
  }
  
  private double getValue(EditText edit)
  {
    return Double.parseDouble(edit.getText().toString()); 
  }

  private void setValue(EditText edit, double value)
  {
    updating = true;
    try
    {
      NumberFormat nf = NumberFormat.getNumberInstance();

      nf.setMaximumFractionDigits(2);
      edit.setText(nf.format(value));
    }
    finally
    {
      updating = false;
    }
  }
  
  private double getMetricValue()
  {
    return getValue(metricEdit); 
  }

  private void setMetricValue(double value)
  {
    setValue(metricEdit, value);
  }
  
  private double getUsValue()
  {
    return getValue(usEdit); 
  }

  private void setUsValue(double value)
  {
    setValue(usEdit, value);
  }

  private void setAdapter(Spinner spinner, int array)
  {
    ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, array, android.R.layout.simple_spinner_item);
    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    spinner.setAdapter(adapter);
  }

  private void typeChanged(int value)
  {
    if (value == 0)
    {
      setAdapter(metricSpinner, R.array.lengthMetricArray);
      setAdapter(usSpinner, R.array.lengthUsArray);
    }
    else
    {
      setAdapter(metricSpinner, R.array.temperatureMetricArray);
      setAdapter(usSpinner, R.array.temperatureUsArray);
    }
  }

  private void metricChanged()
  {
    if (updating || (metricEdit.getText().toString().length() == 0))
      return;

    switch (getQuantity())
    {
      case Common.LENGTH:
        setUsValue(Common.metricLengthToUsLength(getMetricValue(), getMetricLength(), getUsLength()));
        break;

      case Common.TEMPERATURE:
        setUsValue(Common.metricTemperatureToUsTemperature(getMetricValue(), getMetricTemperature(), getUsTemperature()));
        break;
    }
  }

  private void usChanged()
  {
    if (updating || (usEdit.getText().toString().length() == 0))
      return;

    switch (getQuantity())
    {
      case Common.LENGTH:
        setMetricValue(Common.usLengthToMetricLength(getUsValue(), getUsLength(), getMetricLength()));
        break;

      case Common.TEMPERATURE:
        setMetricValue(Common.usTemperatureToMetricTemperature(getUsValue(), getUsTemperature(), getMetricTemperature()));
        break;
    }
  }
  
  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) 
  {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);
    res = getResources();

    button = (Button)findViewById(R.id.aboutButton);
    button.setOnClickListener(this);

    // Type
    ArrayAdapter<CharSequence> typeAdapter = ArrayAdapter.createFromResource(this, R.array.typeArray, android.R.layout.simple_spinner_item);
    typeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

    typeSpinner = (Spinner)findViewById(R.id.typeSpinner);
    typeSpinner.setAdapter(typeAdapter);
    typeSpinner.setOnItemSelectedListener(this);

    // Metric
    metricSpinner = (Spinner)findViewById(R.id.metricSpinner);
    metricSpinner.setOnItemSelectedListener(this);

    metricEdit = (EditText)findViewById(R.id.metricEdit);
    metricEdit.setOnKeyListener(this);

    // US
    usSpinner = (Spinner)findViewById(R.id.usSpinner);
    usSpinner.setOnItemSelectedListener(this);

    usEdit = (EditText)findViewById(R.id.usEdit);
    usEdit.setOnKeyListener(this);
  }
  
  public void onItemSelected(AdapterView<?> parent, View view, int position, long id) 
  {
    if (parent == typeSpinner)
      typeChanged(position);
    else if (parent == metricSpinner)
      metricChanged();
    else if (parent == usSpinner)
      usChanged();
  }
  
  public void onNothingSelected(AdapterView<?> parent)
  {
  }
  
  public boolean onKey(View v, int keyCode, KeyEvent event)
  {
    if (v == metricEdit)
      metricChanged();
    else if (v == usEdit)
      usChanged();

    return true;
  }
  
  public void onClick(View v) 
  {
    Toast.makeText(getApplicationContext(), res.getString(R.string.aboutMessage), Toast.LENGTH_SHORT).show();
  }  
}