unit Common;

interface

const
  MILE_AS_METERS = 1609.3;
  YARD_AS_METERS = 0.9144;

  INCH_IN_FEET = 12;
  FEET_IN_YARD = 3;
  YARDS_IN_MILE = 1760;

  CELSIUS_OFFSET = 273.15;
  FAHRENHEIT_OFFSET = 32;
  FAHRENHEIT_FACTOR = 1.8;

type
  TQuantity =
  (
    qaLength,
    qaTemperature
  );

  TLaMetricLength =
  (
    mlKm,
    mlM,
    mlCm,
    mlMm
  );

  TLaUsLength =
  (
    ulMile,
    ulYard,
    ulFoot,
    ulInch
  );

  TLaMetricTemperature =
  (
    mtCelsius,
    mtKelvin
  );

  TLaUsTemperature =
  (
    utFahrenheit
  );

function MetricLengthToUsLength(
  value: Double;
  metricLength: TLaMetricLength;
  usLength: TLaUsLength): Double;

function UsLengthToMetricLength(
  value: Double;
  usLength: TLaUsLength;
  metricLength: TLaMetricLength): Double;

function MetricTemperatureToUsTemperature(
  value: Double;
  metricTemperature: TLaMetricTemperature;
  usTemperature: TLaUsTemperature): Double;

function UsTemperatureToMetricTemperature(
  value: Double;
  usTemperature: TLaUsTemperature;
  metricTemperature: TLaMetricTemperature): Double;

implementation

function MetricLengthToUsLength(
  value: Double;
  metricLength: TLaMetricLength;
  usLength: TLaUsLength): Double;
var
  meters, yards: Double;
begin
  case metricLength of
    mlKm: meters := 1000*value;
    mlCm: meters := value/100;
    mlMm: meters := value/1000;
  else
    meters := value;
  end;

  yards := meters/YARD_AS_METERS;

  case usLength of
    ulMile: Result := yards/YARDS_IN_MILE;
    ulFoot: Result := FEET_IN_YARD*yards;
    ulInch: Result := FEET_IN_YARD*INCH_IN_FEET*yards;
  else
    Result := yards;
  end;
end;

function UsLengthToMetricLength(
  value: Double;
  usLength: TLaUsLength;
  metricLength: TLaMetricLength): Double;
var
  meters, yards: Double;
begin
  case usLength of
    ulMile: yards := YARDS_IN_MILE*value;
    ulFoot: yards := value/FEET_IN_YARD;
    ulInch: yards := value/(FEET_IN_YARD*INCH_IN_FEET);
  else
    yards := value;
  end;

  meters := YARD_AS_METERS*yards;

  case metricLength of
    mlKm: Result := meters/1000;
    mlCm: Result := 100*meters;
    mlMm: Result := 1000*meters;
  else
    Result := meters;
  end;
end;

function MetricTemperatureToUsTemperature(
  value: Double;
  metricTemperature: TLaMetricTemperature;
  usTemperature: TLaUsTemperature): Double;
var
  celsius, fahrenheit: Double;
begin
  case metricTemperature of
    mtKelvin: celsius := value - CELSIUS_OFFSET;
  else
    celsius := value;
  end;

  fahrenheit := FAHRENHEIT_FACTOR*celsius + FAHRENHEIT_OFFSET;

  Result := fahrenheit;
end;

function UsTemperatureToMetricTemperature(
  value: Double;
  usTemperature: TLaUsTemperature;
  metricTemperature: TLaMetricTemperature): Double;
var
  celsius, fahrenheit: Double;
begin
  fahrenheit := value;

  celsius := (fahrenheit - FAHRENHEIT_OFFSET)/FAHRENHEIT_FACTOR;

  case metricTemperature of
    mtKelvin: Result := celsius + CELSIUS_OFFSET;
  else
    Result := celsius;
  end;
end;

end.
