import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

public class MainFrame extends JFrame
{
  private ResourceBundle res = ResourceBundle.getBundle("Converter");
  private boolean updating = false;

  JPanel contentPane;
  GridLayout gridLayout = new GridLayout();
  TitledBorder titledBorder1 = new TitledBorder("");
  JLabel typeLabel = new JLabel();
  JLabel metricUnitLabel = new JLabel();
  JComboBox metricUnitCombo = new JComboBox();
  JLabel metricAmountLabel = new JLabel();
  JTextField metricAmountField = new JTextField();
  JComboBox typeCombo = new JComboBox();
  JMenuBar menuBar = new JMenuBar();
  JMenu fileMenu = new JMenu();
  JMenuItem exitMenuItem = new JMenuItem();
  JMenu helpMenu = new JMenu();
  JMenuItem aboutMenuItem = new JMenuItem();
  JLabel usUnitLabel = new JLabel();
  JComboBox usUnitCombo = new JComboBox();
  JLabel usAmountLabel = new JLabel();
  JTextField usAmountField = new JTextField();

  public MainFrame()
  {
    try
    {
      setDefaultCloseOperation(EXIT_ON_CLOSE);
      jbInit();

      typeCombo.addItem(res.getString("Length"));
      typeCombo.addItem(res.getString("Temperature"));
    }
    catch (Exception exception)
    {
      exception.printStackTrace();
    }
  }

  private void jbInit() throws Exception
  {
    contentPane = (JPanel) getContentPane();
    contentPane.setLayout(gridLayout);
    this.setJMenuBar(menuBar);
    setSize(new Dimension(247, 290));
    setTitle(res.getString("Converter"));
    contentPane.setBorder(null);
    contentPane.setPreferredSize(new Dimension(0, 0));
    gridLayout.setColumns(1);
    gridLayout.setHgap(2);
    gridLayout.setRows(10);
    typeLabel.setText(res.getString("Type"));
    metricUnitLabel.setText(res.getString("Metric_unit"));
    metricAmountLabel.setText(res.getString("Metric_amount"));
    usUnitLabel.setText(res.getString("US_unit"));
    usAmountLabel.setText(res.getString("US_amount"));
    fileMenu.setText(res.getString("File"));
    exitMenuItem.setText(res.getString("Exit"));
    exitMenuItem.addActionListener(new MainFrame_exitMenuItem_actionAdapter(this));
    helpMenu.setText(res.getString("Help"));
    aboutMenuItem.setText(res.getString("About"));
    aboutMenuItem.addActionListener(new MainFrame_aboutMenuItem_actionAdapter(this));
    typeCombo.addActionListener(new MainFrame_typeCombo_actionAdapter(this));
    metricAmountField.addActionListener(new MainFrame_metricField_actionAdapter(this));
    usAmountField.addActionListener(new MainFrame_usField_actionAdapter(this));
    contentPane.add(typeLabel);
    contentPane.add(typeCombo);
    contentPane.add(metricUnitLabel);
    contentPane.add(metricUnitCombo);
    contentPane.add(metricAmountLabel);
    contentPane.add(metricAmountField);
    contentPane.add(usUnitLabel);
    contentPane.add(usUnitCombo);
    contentPane.add(usAmountLabel);
    contentPane.add(usAmountField);
    menuBar.add(fileMenu);
    menuBar.add(helpMenu);
    fileMenu.add(exitMenuItem);
    helpMenu.add(aboutMenuItem);
  }

  private String getMetricLength(int value)
  {
    switch (value)
    {
      case Common.KM:
        return res.getString("Kilometers");

      case Common.CM:
        return res.getString("Centimeters");

      case Common.MM:
        return res.getString("Millimeters");

      default:
        return res.getString("Meters");
    }
  }

  private String getUsLength(int value)
  {
    switch (value)
    {
      case Common.MILE:
        return res.getString("Miles");

      case Common.FOOT:
        return res.getString("Feet");

      case Common.INCH:
        return res.getString("Inches");

      default:
        return res.getString("Yards");
    }
  }

  private String getMetricTemperature(int value)
  {
    switch (value)
    {
      case Common.KELVIN:
        return res.getString("Kelvin");

      default:
        return res.getString("Celsius");
    }
  }

  private String getUsTemperature(int value)
  {
    return res.getString("Fahrenheit");
  }

  private int getQuantity()
  {
    return typeCombo.getSelectedIndex();
  }

  private int getMetricLength()
  {
    return metricUnitCombo.getSelectedIndex();
  }

  private int getUsLength()
  {
    return usUnitCombo.getSelectedIndex();
  }

  private int getMetricTemperature()
  {
    return metricUnitCombo.getSelectedIndex();
  }

  private int getUsTemperature()
  {
    return usUnitCombo.getSelectedIndex();
  }

  private double getValue(JTextField field)
  {
    try
    {
      return Double.valueOf(field.getText().trim()).intValue();
    }
    catch (Exception e)
    {
      return 0;
    }
  }

  private void setValue(JTextField field, double value)
  {
    updating = true;
    try
    {
      NumberFormat nf = NumberFormat.getNumberInstance();

      nf.setMaximumFractionDigits(2);
      field.setText(nf.format(value));
    }
    finally
    {
      updating = false;
    }
  }

  private double getMetricValue()
  {
    return getValue(metricAmountField);
  }

  private void setMetricValue(double value)
  {
    setValue(metricAmountField, value);
  }

  private double getUsValue()
  {
    return getValue(usAmountField);
  }

  private void setUsValue(double value)
  {
    setValue(usAmountField, value);
  }

  public void exitMenuItem_actionPerformed(ActionEvent e)
  {
    System.exit(0);
  }

  public void aboutMenuItem_actionPerformed(ActionEvent e)
  {
    JOptionPane.showMessageDialog(
      this,
      res.getString("Converter_is"),
      res.getString("About_Converter"),
      JOptionPane.INFORMATION_MESSAGE);
  }

  public void typeCombo_actionPerformed(ActionEvent e)
  {
    metricUnitCombo.removeAllItems();
    usUnitCombo.removeAllItems();

    switch (getQuantity())
    {
      case Common.LENGTH:
        for (int i = 0; i < 4; i++)
      {
        metricUnitCombo.addItem(getMetricLength(i));
      }

      for (int i = 0; i < 4; i++)
      {
        usUnitCombo.addItem(getUsLength(i));
      }
      break;

      case Common.TEMPERATURE:
        for (int i = 0; i < 2; i++)
      {
        metricUnitCombo.addItem(getMetricTemperature(i));
      }

      for (int i = 0; i < 1; i++)
      {
        usUnitCombo.addItem(getUsTemperature(i));
      }
      break;
    }

    metricUnitCombo.setSelectedIndex(0);
    usUnitCombo.setSelectedIndex(0);
  }

  public void metricField_actionPerformed(ActionEvent e)
  {
    if (updating || (metricAmountField.getText() == ""))
      return;

    switch (getQuantity())
    {
      case Common.LENGTH:
        setUsValue(Common.metricLengthToUsLength(getMetricValue(), getMetricLength(), getUsLength()));
        break;

      case Common.TEMPERATURE:
        setUsValue(Common.metricTemperatureToUsTemperature(getMetricValue(), getMetricTemperature(), getUsTemperature()));
        break;
    }
  }

  public void usField_actionPerformed(ActionEvent e)
  {
    if (updating || (usAmountField.getText() == ""))
      return;

    switch (getQuantity())
    {
      case Common.LENGTH:
        setMetricValue(Common.usLengthToMetricLength(getUsValue(), getUsLength(), getMetricLength()));
        break;

      case Common.TEMPERATURE:
        setMetricValue(Common.usTemperatureToMetricTemperature(getUsValue(), getUsTemperature(), getMetricTemperature()));
        break;
    }
  }
}

class MainFrame_aboutMenuItem_actionAdapter implements ActionListener
{
  private MainFrame adaptee;
  MainFrame_aboutMenuItem_actionAdapter(MainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.aboutMenuItem_actionPerformed(e);
  }
}

class MainFrame_usField_actionAdapter implements ActionListener
{
  private MainFrame adaptee;

  MainFrame_usField_actionAdapter(MainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.usField_actionPerformed(e);
  }
}

class MainFrame_metricField_actionAdapter implements ActionListener
{
  private MainFrame adaptee;

  MainFrame_metricField_actionAdapter(MainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.metricField_actionPerformed(e);
  }
}

class MainFrame_typeCombo_actionAdapter implements ActionListener
{
  private MainFrame adaptee;

  MainFrame_typeCombo_actionAdapter(MainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.typeCombo_actionPerformed(e);
  }
}

class MainFrame_exitMenuItem_actionAdapter implements ActionListener
{
  private MainFrame adaptee;

  MainFrame_exitMenuItem_actionAdapter(MainFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.exitMenuItem_actionPerformed(e);
  }
}
