﻿using System;
using System.Windows;
using System.Windows.Controls;
using Sisulizer; // Contains conversion functions

namespace Converter
{
  public partial class Page : UserControl
  {
    private bool updating = false;

    public Page()
    {
      InitializeComponent();
    }

    public TQuantity Quantity
    {
      get
      {
        return (TQuantity)typeCombo.SelectedIndex;
      }
    }

    public TLaMetricLength MetricLength
    {
      get
      {
        return (TLaMetricLength)metricCombo.SelectedIndex;
      }
    }

    public TLaUsLength UsLength
    {
      get
      {
        return (TLaUsLength)usCombo.SelectedIndex;
      }
    }

    public TLaMetricTemperature MetricTemperature
    {
      get
      {
        return (TLaMetricTemperature)metricCombo.SelectedIndex;
      }
    }

    public TLaUsTemperature UsTemperature
    {
      get
      {
        return (TLaUsTemperature)usCombo.SelectedIndex;
      }
    }

    public double MetricValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(metricText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        updating = true;
        metricText.Text = value.ToString("F");
      }
    }

    public double UsValue
    {
      get
      {
        try
        {
          return Convert.ToDouble(usText.Text);
        }
        catch
        {
          return 0;
        }
      }

      set
      {
        updating = true;
        usText.Text = value.ToString("F");
      }
    }

    private void addMetric(string value)
    {
      metricCombo.Items.Add(value);
    }

    private void addUs(string value)
    {
      usCombo.Items.Add(value);
    }

    private void WindowLoaded(object sender, RoutedEventArgs e)
    {
      typeCombo.Items.Add("Length");
      typeCombo.Items.Add("Temperature");
      typeCombo.SelectedIndex = 0;
    }

    public void TypeChanged(object sender, SelectionChangedEventArgs e)
    {
      metricCombo.Items.Clear();
      usCombo.Items.Clear();

      switch (Quantity)
      {
        case TQuantity.qaLength:
          addMetric("Kilometers");
          addMetric("Meters");
          addMetric("Centimeters");
          addMetric("Millimeters");
          addUs("Miles");
          addUs("Yards");
          addUs("Feet");
          addUs("Inches");
          break;

        case TQuantity.qaTemperature:
          addMetric("Celsius");
          addMetric("Kelvin");
          addUs("Fahrenheit");
          break;
      }

      metricCombo.SelectedIndex = 0;
      usCombo.SelectedIndex = 0;
    }

    private void MetricTextChanged(object sender, TextChangedEventArgs e)
    {
      if (updating || (metricText.Text == ""))
      {
        updating = false;
        return;
      }

      switch (Quantity)
      {
        case TQuantity.qaLength:
          UsValue = Measurements.metricLengthToUsLength(MetricValue, MetricLength, UsLength);
          break;

        case TQuantity.qaTemperature:
          UsValue = Measurements.metricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
          break;
      }
    }

    private void UsTextChanged(object sender, TextChangedEventArgs e)
    {
      if (updating || (usText.Text == ""))
      {
        updating = false;
        return;
      }

      switch (Quantity)
      {
        case TQuantity.qaLength:
          MetricValue = Measurements.usLengthToMetricLength(UsValue, UsLength, MetricLength);
          break;

        case TQuantity.qaTemperature:
          MetricValue = Measurements.usTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
          break;
      }
    }
  }
}
