/*
  Copyrights 2005-2011 Sisulizer

  A class that contains conversion functions between metric and US system.
  This code is common for Windows Forms and WPF.
*/

namespace Sisulizer
{
  public enum TQuantity {qaLength, qaTemperature};
  public enum TLaMeasurementSystem {msMetric, msUS};
  public enum TLaMetricLength {mlKm, mlM, mlCm, mlMm};
  public enum TLaUsLength {ulMile, ulYard, ulFoot, ulInch};
  public enum TLaMetricTemperature {mtCelsius, mtKelvin};
  public enum TLaUsTemperature {utFahrenheit};

  public class Measurements
  {
    public const double MILE_AS_METERS = 1609.3f;
    public const double YARD_AS_METERS = 0.9144f;

    public const int INCH_IN_FEET = 12;
    public const int FEET_IN_YARD = 3;
    public const int YARDS_IN_MILE = 1760;

    public const double CELSIUS_OFFSET = 273.15f;

    public static TLaMeasurementSystem measurementSystem;

    public static double metricLengthToUsLength(
      double value,
      TLaMetricLength metricLength,
      TLaUsLength usLength)
    {
      double meters;
      
      switch (metricLength)
      {
        case TLaMetricLength.mlKm: 
          meters = 1000*value;
          break;
          
        case TLaMetricLength.mlCm: 
          meters = value/100;
          break;
          
        case TLaMetricLength.mlMm: 
          meters = value/1000;
          break;
          
        default:
         meters = value;
         break;
      }  

      double yards = meters/YARD_AS_METERS;

      switch (usLength)
      {
        case TLaUsLength.ulMile: 
          return yards/YARDS_IN_MILE;
          
        case TLaUsLength.ulFoot: 
          return FEET_IN_YARD*yards;
          
        case TLaUsLength.ulInch: 
          return FEET_IN_YARD*INCH_IN_FEET*yards;
          
        default: 
          return yards;
      }
    }  

    public static double usLengthToMetricLength(
      double value,
      TLaUsLength usLength,
      TLaMetricLength metricLength)
    {
      double yards;
      
      switch (usLength)
      {
        case TLaUsLength.ulMile: 
          yards = YARDS_IN_MILE*value;
          break;
          
        case TLaUsLength.ulFoot: 
          yards = value/FEET_IN_YARD;
          break;
          
        case TLaUsLength.ulInch: 
          yards = value/(FEET_IN_YARD*INCH_IN_FEET);
          break;
          
        default:
          yards = value;
          break;
      }    

      double meters = YARD_AS_METERS*yards;

      switch (metricLength)
      {
        case TLaMetricLength.mlKm: 
          return meters/1000;
          
        case TLaMetricLength.mlCm: 
          return 100*meters;
          
        case TLaMetricLength.mlMm: 
          return 1000*meters;
          
        default:
          return meters;
      }   
    }  

    public static double metricTemperatureToUsTemperature(
      double value,
      TLaMetricTemperature metricTemperature,
      TLaUsTemperature usTemperature)
    {
      double celsius;
      
      switch (metricTemperature)
      {
        case TLaMetricTemperature.mtKelvin: 
          celsius = value - CELSIUS_OFFSET;
          break;
          
        default:
          celsius = value;
          break;
      }    

      double fahrenheit = 1.8*celsius + 32;

      return fahrenheit;
    }  

    public static double usTemperatureToMetricTemperature(
      double value,
      TLaUsTemperature usTemperature,
      TLaMetricTemperature metricTemperature)
    {
      double fahrenheit = value;
      
      double celsius = (fahrenheit -32)/1.8;

      switch (metricTemperature)
      {
        case TLaMetricTemperature.mtKelvin: 
          return celsius + CELSIUS_OFFSET;

        default:
          return celsius;
      }    
    }  
  }
}
