﻿namespace Converter;

interface

uses
  System.Drawing,
  System.Collections,
  System.Collections.Generic,
  System.Windows.Forms,
  System.ComponentModel,
  Sisulizer;

type
  /// <summary>
  /// Summary description for MainForm.
  /// </summary>
  MainForm = partial class(System.Windows.Forms.Form)
  private
    FUpdating: Boolean;

    function GetQuantity: TQuantity;
    function GetMetricLength: TLaMetricLength;
    function GetUsLength: TLaUsLength;
    function GetMetricTemperature: TLaMetricTemperature;
    function GetUsTemperature: TLaUsTemperature;
    function GetMetricValue: Double;
    function GetUsValue: Double;

    procedure SetMetricValue(value: Double);
    procedure SetUsValue(value: Double);

    procedure AddMetric(const value: String);
    procedure AddUs(const value: String);

    method MainForm_Load(sender: System.Object; e: System.EventArgs);
    method typeCombo_SelectedIndexChanged(sender: System.Object; e: System.EventArgs);
    method metricText_TextChanged(sender: System.Object; e: System.EventArgs);
    method usText_TextChanged(sender: System.Object; e: System.EventArgs);
    method exitToolStripMenuItem_Click(sender: System.Object; e: System.EventArgs);
    method aboutToolStripMenuItem_Click(sender: System.Object; e: System.EventArgs);

  protected
    method Dispose(disposing: Boolean); override;

  public
    constructor;

    property Quantity: TQuantity read GetQuantity;
    property MetricLength: TLaMetricLength read GetMetricLength;
    property UsLength: TLaUsLength read GetUsLength;
    property MetricTemperature: TLaMetricTemperature read GetMetricTemperature;
    property UsTemperature: TLaUsTemperature read GetUsTemperature;
    property MetricValue: Double read GetMetricValue write SetMetricValue;
    property UsValue: Double read GetUsValue write SetUsValue;
  end;

implementation

{$REGION Construction and Disposition}
constructor MainForm;
begin
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent();

  //
  // TODO: Add any constructor code after InitializeComponent call
  //
end;

method MainForm.Dispose(disposing: Boolean);
begin
  if disposing then begin
    if assigned(components) then
      components.Dispose();

    //
    // TODO: Add custom disposition code here
    //
  end;
  inherited Dispose(disposing);
end;
{$ENDREGION}

function MainForm.GetQuantity: TQuantity;
begin
  Result := TQuantity(typeCombo.SelectedIndex);
end;

function MainForm.GetMetricLength: TLaMetricLength;
begin
  Result := TLaMetricLength(metricCombo.SelectedIndex);
end;

function MainForm.GetUsLength: TLaUsLength;
begin
  Result := TLaUsLength(usCombo.SelectedIndex);
end;

function MainForm.GetMetricTemperature: TLaMetricTemperature;
begin
  Result := TLaMetricTemperature(metricCombo.SelectedIndex);
end;

function MainForm.GetUsTemperature: TLaUsTemperature;
begin
  Result := TLaUsTemperature(usCombo.SelectedIndex);
end;

function MainForm.GetMetricValue: Double;
begin
  try
    Result := Convert.ToDouble(metricText.Text);
  except
    Result := 0;
  end;
end;

procedure MainForm.SetMetricValue(value: Double);
begin
  try
    FUpdating := True;
    metricText.Text := value.ToString('F');
  finally
    FUpdating := False;
  end;
end;

function MainForm.GetUsValue: Double;
begin
  Result := Convert.ToDouble(usText.Text);
end;

procedure MainForm.SetUsValue(value: Double);
begin
  try
    FUpdating := True;
    usText.Text := value.ToString('F');
  finally
    FUpdating := False;
  end;
end;

method MainForm.AddMetric(const value: String);
begin
  metricCombo.Items.Add(value);
end;

method MainForm.AddUs(const value: String);
begin
  usCombo.Items.Add(value);
end;

method MainForm.MainForm_Load(sender: System.Object; e: System.EventArgs);
begin
  typeCombo.Items.Clear;
  typeCombo.Items.Add(Properties.Resources.Length);
  typeCombo.Items.Add(Properties.Resources.Temperature);
  typeCombo.SelectedIndex := 0;
end;

method MainForm.typeCombo_SelectedIndexChanged(sender: System.Object; e: System.EventArgs);
begin
  metricCombo.Items.Clear;
  usCombo.Items.Clear;

  case Quantity of
    TQuantity.qaLength:
    begin
      AddMetric(Properties.Resources.Kilometers);
      AddMetric(Properties.Resources.Meters);
      AddMetric(Properties.Resources.Centimeters);
      AddMetric(Properties.Resources.Millimeters);
      AddUs(Properties.Resources.Miles);
      AddUs(Properties.Resources.Yards);
      AddUs(Properties.Resources.Feet);
      AddUs(Properties.Resources.Inches);
    end;

    TQuantity.qaTemperature:
    begin
      AddMetric(Properties.Resources.Celsius);
      AddMetric(Properties.Resources.Kelvin);
      AddUs(Properties.Resources.Fahrenheit);
    end;
  end;

  metricCombo.SelectedIndex := 0;
  usCombo.SelectedIndex := 0;
end;

method MainForm.metricText_TextChanged(sender: System.Object; e: System.EventArgs);
begin
  if FUpdating or (metricText.Text = '') then
    Exit;

  case Quantity of
    TQuantity.qaLength:
      UsValue := Measurements.metricLengthToUsLength(MetricValue, MetricLength, UsLength);

    TQuantity.qaTemperature:
      UsValue := Measurements.metricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
  end;
end;

method MainForm.usText_TextChanged(sender: System.Object; e: System.EventArgs);
begin
  if FUpdating or (usText.Text = '') then
    Exit;

  case Quantity of
    TQuantity.qaLength:
      MetricValue := Measurements.usLengthToMetricLength(UsValue, UsLength, MetricLength);

    TQuantity.qaTemperature:
      MetricValue := Measurements.usTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
  end;
end;

method MainForm.exitToolStripMenuItem_Click(sender: System.Object; e: System.EventArgs);
begin
  Application.Exit;
end;

method MainForm.aboutToolStripMenuItem_Click(sender: System.Object; e: System.EventArgs);
begin
  MessageBox.Show(
    Properties.Resources.About_Message,
    Properties.Resources.About_Converter,
    MessageBoxButtons.OK,
    MessageBoxIcon.Information);
end;

end.
