#include "measurement.h"

double metricLengthToUsLength(
  double value,
  MetricLength metricLength,
  UsLength usLength)
{
  double meters, yards;

  switch (metricLength)
  {
    case Km:
      meters = 1000*value;
      break;

    case cm:
      meters = value/100;
      break;

    case mm:
      meters = value/1000;
      break;

    default:
      meters = value;
  }

  yards = meters/YARD_AS_METERS;

  switch (usLength)
  {
    case Mile:
      return yards/YARDS_IN_MILE;

    case Foot:
      return FEET_IN_YARD*yards;

    case Inch:
      return FEET_IN_YARD*INCH_IN_FEET*yards;

    default:
      return yards;
  }
}

double usLengthToMetricLength(
  double value,
  UsLength usLength,
  MetricLength metricLength)
{
  double meters, yards;

  switch (usLength)
  {
    case Mile:
      yards = YARDS_IN_MILE*value;
      break;

    case Foot:
      yards = value/FEET_IN_YARD;
      break;

    case Inch:
      yards = value/(FEET_IN_YARD*INCH_IN_FEET);
      break;

    default:
      yards = value;
  }

  meters = YARD_AS_METERS*yards;

  switch (metricLength)
  {
    case Km:
      return meters/1000;
      break;

    case cm:
      return 100*meters;
      break;

    case mm:
      return 1000*meters;
      break;

    default:
      return meters;
  }
}

double metricTemperatureToUsTemperature(
  double value,
  MetricTemperature metricTemperature,
  UsTemperature usTemperature)
{
  double celsius;

  switch (metricTemperature)
  {
    case Kelvin:
      celsius = value - CELSIUS_OFFSET;
      break;

    default:
      celsius = value;
  }

  return FAHRENHEIT_FACTOR*celsius + FAHRENHEIT_OFFSET;
}

double usTemperatureToMetricTemperature(
  double value,
  UsTemperature usTemperature,
  MetricTemperature metricTemperature)
{
  double fahrenheit = value;
  double celsius = (fahrenheit - FAHRENHEIT_OFFSET)/FAHRENHEIT_FACTOR;

  switch (metricTemperature)
  {
    case Kelvin:
      return celsius + CELSIUS_OFFSET;

    default:
      return celsius;
  }
}
