/*
 * This application is written in C++Builder XE2. To compile it with another
 * C++Builder version read ..\..\Readme.html
 */

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "resource.h"
#include "LaDialog.hpp"

#pragma package(smart_init)
#pragma link "LaCommon"
#pragma link "LaDialog"
#pragma resource "*.dfm"

TMainForm *MainForm;

__fastcall TMainForm::TMainForm(TComponent* Owner): TForm(Owner)
{
  Application->OnHint = DisplayHint;
}

TQuantity TMainForm::GetQuantity()
{
  return (TQuantity)TypeCombo->Items->Objects[TypeCombo->ItemIndex];
}

TLaMetricLength TMainForm::GetMetricLength()
{
  return (TLaMetricLength)MetricCombo->Items->Objects[MetricCombo->ItemIndex];
}

TLaUsLength TMainForm::GetUsLength()
{
  return (TLaUsLength)UsCombo->Items->Objects[UsCombo->ItemIndex];
}

TLaMetricTemperature TMainForm::GetMetricTemperature()
{
  return (TLaMetricTemperature)MetricCombo->Items->Objects[MetricCombo->ItemIndex];
}

TLaUsTemperature TMainForm::GetUsTemperature()
{
  return (TLaUsTemperature)UsCombo->Items->Objects[UsCombo->ItemIndex];
}

Double TMainForm::GetMetricValue()
{
  return StrToFloat(MetricEdit->Text);
}

void TMainForm::SetMetricValue(Double value)
{
  FUpdating = true;
  MetricEdit->Text = FloatToStrF(value, ffNumber, 10, 2);
  FUpdating = false;
}

Double TMainForm::GetUsValue()
{
  return StrToFloat(UsEdit->Text);
}

void TMainForm::SetUsValue(Double value)
{
  FUpdating = true;
  UsEdit->Text = FloatToStrF(value, ffNumber, 10, 2);
  FUpdating = false;
}

void __fastcall TMainForm::DisplayHint(TObject *sender)
{
  // Shows the hint on the status bar

  StatusBar->SimpleText = GetLongHint(Application->Hint);
}

AnsiString GetQuantityStr(TQuantity value)
{
  switch (value)
  {
    case qaLength:
      return LoadStr(SLength);
  default:
    return LoadStr(STemperature);
  }
}

void __fastcall TMainForm::Initialize()
{
  TypeCombo->Clear();

  for (int q = qaLength; q <= qaTemperature; q++)
	TypeCombo->AddItem(GetQuantityStr((TQuantity)q), (TObject*)q);

  TypeCombo->ItemIndex = 0;
  TypeComboChange(this);

  CheckCodePage();
}

void __fastcall TMainForm::AboutMenuClick(TObject *Sender)
{
  ShowMessage(LoadStr(SAboutCaption) + "\r\n\r\n" + LoadStr(SAbout));
}

void TMainForm::AddMetric(AnsiString str, int value)
{
  MetricCombo->AddItem(str, (TObject*)value);
}

void TMainForm::AddUs(AnsiString str, int value)
{
  UsCombo->AddItem(str, (TObject*)value);
}

AnsiString GetMetricLengthStr(TLaMetricLength value)
{
  switch (value)
  {
    case mlKm: return LoadStr(SKilometers);
    case mlCm: return LoadStr(SCentimeters);
    case mlMm: return LoadStr(SMillimeters);
  default:
    return LoadStr(SMeters);
  }
}

AnsiString GetUsLengthStr(TLaUsLength value)
{
  switch (value)
  {
    case ulMile: return LoadStr(SMiles);
    case ulFoot: return LoadStr(SFeet);
    case ulInch: return LoadStr(SInches);
  default:
    return LoadStr(SYards);
  }
}

AnsiString GetMetricTemperatureStr(TLaMetricTemperature value)
{
  switch (value)
  {
    case mtKelvin: return LoadStr(SKelvin);
  default:
    return LoadStr(SCelsius);
  }
}

AnsiString GetUsTemperatureStr(TLaUsTemperature value)
{
  return LoadStr(SFahrenheit);
}

void __fastcall TMainForm::TypeComboChange(TObject *Sender)
{
  MetricCombo->Items->Clear();
  UsCombo->Items->Clear();

  switch (Quantity)
  {
	case qaLength:
	{
	  for (int i = mlKm; i <= mlMm; i++)
		AddMetric(GetMetricLengthStr((TLaMetricLength)i), i);

	  for (int i = ulMile; i <= ulInch; i++)
		AddUs(GetUsLengthStr((TLaUsLength)i), i);

	  break;
	}

	case qaTemperature:
	{
	  for (int i = mtCelsius; i <= mtKelvin; i++)
		AddMetric(GetMetricTemperatureStr((TLaMetricTemperature)i), i);

	  for (int i = utFahrenheit; i <= utFahrenheit; i++)
        AddUs(GetUsTemperatureStr((TLaUsTemperature)i), i);

      break;
    }
  }

  MetricCombo->ItemIndex = 0;
  UsCombo->ItemIndex = 0;
}

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  FUpdating = false;
  Initialize();
}

void __fastcall TMainForm::MetricEditChange(TObject *Sender)
{
  if (FUpdating || (MetricEdit->Text.Length() == 0))
    return;

  switch (Quantity)
  {
    case qaLength:
      UsValue = MetricLengthToUsLength(MetricValue, MetricLength, UsLength);
      break;

    case qaTemperature:
      UsValue = MetricTemperatureToUsTemperature(MetricValue, MetricTemperature, UsTemperature);
      break;
  }
}

void __fastcall TMainForm::UsEditChange(TObject *Sender)
{
  if (FUpdating || (UsEdit->Text.Length() == 0))
    return;

  switch (Quantity)
  {
    case qaLength:
      MetricValue = UsLengthToMetricLength(UsValue, UsLength, MetricLength);
      break;

    case qaTemperature:
      MetricValue = UsTemperatureToMetricTemperature(UsValue, UsTemperature, MetricTemperature);
      break;
  }
}

void __fastcall TMainForm::ExitMenuClick(TObject *Sender)
{
  Close();
}

void __fastcall TMainForm::LanguageMenuClick(TObject *Sender)
{
  CheckResourceCount();

  if (SelectResourceLocale("EN"))
  {
	SetCurrentDefaultLocaleReg();
    Initialize();
  }
}

void __fastcall TMainForm::SetDefaultLanguageMenuClick(TObject *Sender)
{
  if (ClearCurrentDefaultLocaleReg)
  {
    SetNewResourceFile();
    Initialize();
  }
}

