{
  Copyrights 2005-2008 Sisulizer Ltd.

  This samples shows how to localized both UI and database and how to
  use only localized data from database.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, Grids, DB, DBGrids, ADODB;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    DatabaseMenu: TMenuItem;
    OpenMenu: TMenuItem;
    CloseMenu: TMenuItem;
    N1: TMenuItem;
    ExitMenu: TMenuItem;
    HelpMenu: TMenuItem;
    LanguageMenu: TMenuItem;
    N2: TMenuItem;
    AboutMenu: TMenuItem;
    DBGrid1: TDBGrid;
    Connection1: TADOConnection;
    Query1: TADOQuery;
    DataSource1: TDataSource;
    procedure FormShow(Sender: TObject);
    procedure DatabaseMenuClick(Sender: TObject);
    procedure OpenMenuClick(Sender: TObject);
    procedure CloseMenuClick(Sender: TObject);
    procedure ExitMenuClick(Sender: TObject);
    procedure LanguageMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);

  private
    procedure Initialize;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  LaResource,
  LaDbUtils,
  LaDialog,
  LaTranslator;

procedure TMainForm.Initialize;

  function GetLanguage: String;
  begin
    if LoadedResourceLocale = '' then
      Result := 'en'
    else
      Result := LocaleToIso639(ExtensionToLocale(LoadedResourceLocale));
  end;

resourcestring
  SId = 'Id';
  SName = 'Name';
  SPopulation = 'Population';
  SCapital = 'Capital';
  SDescription = 'Description';
begin
  Connection1.Connected := True;

  // Create a query that shows only those rows that has matching lang field
  Query1.Close;
  Query1.SQL.Clear;
  Query1.SQL.Add(Format('SELECT * FROM Country WHERE Lang=''%s''', [GetLanguage]));
  Query1.Open;

  // Hide the language field
  HideField(Query1, 'Language');

  // Sets the display labels of the fields.
  SetFieldDisplayLabel(Query1, 'Id', SId);
  SetFieldDisplayLabel(Query1, 'Name', SName);
  SetFieldDisplayLabel(Query1, 'Population', SPopulation);
  SetFieldDisplayLabel(Query1, 'Capital', SCapital);
  SetFieldDisplayLabel(Query1, 'Description', SDescription);

  DataSource1.DataSet := Query1;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  OpenMenuClick(Self);
end;

procedure TMainForm.OpenMenuClick(Sender: TObject);
begin
  Initialize;
end;

procedure TMainForm.CloseMenuClick(Sender: TObject);
begin
  Query1.Close;
end;

procedure TMainForm.ExitMenuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.DatabaseMenuClick(Sender: TObject);
begin
  OpenMenu.Enabled := not Query1.Active;
  CloseMenu.Enabled := Query1.Active;
end;

procedure TMainForm.LanguageMenuClick(Sender: TObject);
begin
  CheckResourceCount;

  if SelectResourceLocale('EN', [], [roSaveLocale]) then
  begin
    Initialize;
  end;
end;

procedure TMainForm.AboutMenuClick(Sender: TObject);
resourcestring
  SAbout = 'This application demonstrates how to localize database content.';
begin
  ShowMessage(SAbout);
end;

initialization
  // Translate only string properties when performing runtime language switch
  LaEnabledProperties := STRING_TYPES_C;
end.
