{
  This applications shows how to localize and use custom resources that
  contain plain text, defined text, plain binary, defined binary, image, HTML,
  and XML resources.

  The custom resource data is defined in Data.rc resource file.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, OleCtrls, SHDocVw;

type
  TForm1 = class(TForm)
    PageControl: TPageControl;
    StringsSheet: TTabSheet;
    StringsLabel: TLabel;
    PlainLabel: TLabel;
    PlainValue: TLabel;
    CombinedLabel: TLabel;
    CombinedValue: TLabel;
    XmlLabel: TLabel;
    XmlValue: TLabel;
    HtmlLabel: TLabel;
    HtmlValue: TLabel;
    ImageSheet: TTabSheet;
    Image1: TImage;
    ImageLabel: TLabel;
    ImageCombo: TComboBox;
    XmlSheet: TTabSheet;
    HtmlSheet: TTabSheet;
    WebBrowser1: TWebBrowser;
    HtmlResourceLabel: TLabel;
    HtmlDataSheet: TTabSheet;
    Label1: TLabel;
    XmlDataLabel: TLabel;
    TextSheet: TTabSheet;
    TextLabel: TLabel;
    TextDataLabel: TLabel;
    XmlResourceLabel: TLabel;
    XmlCombo: TComboBox;
    HtmlDataMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure ImageComboChange(Sender: TObject);
    procedure XmlComboChange(Sender: TObject);

  private
    function AllocStream(const resourceType, resourceName: String): TResourceStream;

    function LoadText(const resourceType, resourceName: String): String;
    procedure PopulateCombo(comboBox: TComboBox; const resourceType: String);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  Jpeg,
  LaResource;

const
  IMAGE_RES_C = 'IMAGEDATA';
  XML_RES_C = 'XMLDATA';

resourcestring
  // This string contains plain text
  SPlain = 'Plain string';

  // This string contains structured data. Each items is separated by ;
  // Sisulizer uses regular expression to parse the data.
  SExpression = 'One;Two;Three';

  // This string contains XML data
  SXml =
    '<?xml version="1.0" encoding="UTF-8"?>'#13#10 +
    '<sample>'#13#10 +
    '  <plain>Hello world</plain>'#13#10 +
    '</sample>';

  // This string contains HTML data
  SHtml =
    '<html>'#13#10 +
    '  <head>'#13#10 +
    '    <title>Countries</title>'#13#10 +
    '  </head>'#13#10 +
    '  <body>'#13#10 +
    '    <h1>Countries</h1>'#13#10 +
    '    <p>This is a sample</p>'#13#10 +
    '  </body>'#13#10 +
    '</html>';


function TForm1.AllocStream(const resourceType, resourceName: String): TResourceStream;
begin
  Result := TResourceStream.Create(HInstance, PChar(resourceName), PChar(resourceType));

  // If you plan to implemnt runtime language switch do not use HInstance but
  // LaResource.GetResourceInstance because HInstance always loads from
  // the main EXE. GetResourceInstance returns instance of resource DLL if it
  // has been loaded.
  //Result := TResourceStream.Create(GetResourceInstance, PChar(resourceName), PChar(resourceType));
end;

procedure TForm1.PopulateCombo(comboBox: TComboBox; const resourceType: String);

  function EnumResourceNamesProc(
    module: THandle;
    resType: PChar;
    resName: PChar;
    comboBox: TComboBox): Bool; stdcall;
  begin
    comboBox.Items.Add(resName);
    Result := True;
  end;

begin
  // Enum IMAGEDATA resource names and adds them to the combo box
  EnumResourceNames(GetResourceInstance, PChar(resourceType), @EnumResourceNamesProc, Integer(comboBox));
  comboBox.ItemIndex := 0;

  if Assigned(comboBox.OnChange) then
    comboBox.OnChange(Self);
end;

function TForm1.LoadText(const resourceType, resourceName: String): String;
var
  str: Utf8String;
  stream: TResourceStream;
begin
  // Load a custom resource as UTF-8 string and show it in the label
  stream := AllocStream(resourceType, resourceName);
  try
    SetLength(str, stream.Size);
    stream.Read(str[1], stream.Size);

    // Remove UTF-8 header if any
    if (str[1] = Chr($EF)) and (str[2] = Chr($BB)) and (str[3] = Chr($BF)) then
      Delete(str, 1, 3);

    Result := String(str);
  finally
    stream.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  // Strings
  PlainValue.Caption := SPlain;
  CombinedValue.Caption := SExpression;
  XmlValue.Caption := SXml;
  HtmlValue.Caption := SHtml;

  // Images
  PopulateCombo(ImageCombo, IMAGE_RES_C);

  // XML
  PopulateCombo(XmlCombo, XML_RES_C);

  // Text
  TextDataLabel.Caption := LoadText('PLAINTEXT', 'PlainText');

  // Standard Html
  WebBrowser1.Navigate('res://Project1.exe/html');

  // Custom Html
  HtmlDataMemo.Text := LoadText('HTMLDATA', 'Countries');

  PageControl.ActivePageIndex := 0;
end;

procedure TForm1.ImageComboChange(Sender: TObject);
var
  image: TJpegImage;
  stream: TResourceStream;
begin
  // Load a custom image resource and show the image
  stream := AllocStream(IMAGE_RES_C, ImageCombo.Items[ImageCombo.ItemIndex]);
  image := TJpegImage.Create;
  try
    image.LoadFromStream(stream);
    Image1.Picture.Graphic := image;
  finally
    image.Free;
    stream.Free;
  end;
end;

procedure TForm1.XmlComboChange(Sender: TObject);
begin
  // Load a custom XML resource data and show it as text
  XmlDataLabel.Caption := LoadText(XML_RES_C, XmlCombo.Items[XmlCombo.ItemIndex]);
end;

end.
