{
  Copyrights 2005-2009 Sisulizer Ltd.

  This sample shows how to localized Title.Caption of TDBGrid.

  When you use TDBGrid  you have two options. Either you populate
  TDBGrid.Columns manually on design time or you leave it empty and VCL
  populates the column on run time.

  1) Design time columns
  You use Delphi IDE to add the columns that you want to be visible.
  Each TColumn contains Title.Caption property that specifies the caption that
  is shown on the grid. Unfortunately VCL does not save the Caption property
  into DFM if the value of the Caption equals to the name of the field in the
  database. In this sample the field names are plain English and this is why
  even if you add columns manually the values are not saved to DFM

    Columns = <
      item
        Expanded = False
        FieldName = 'Id'
        Width = 30
        Visible = True
      end
      ...

  No Caption property here. To localize captions you have to options.
  - You set the Title.Caption value such that it does not match the field name.
    This way VCL saves it and SL can automatically localize it.
  - You assign the Caption properties on run time using resource strings. This
    sample uses this method. See TMainForm.FormCreate.

  2) Default columns
  You don't populate the columns and this makes VCL to automatically populate
  it on run time. There is not Columns data in DFM so SL can not automatically
  locale the columns. You have to assign the Caption properties on run time.
  See TMainForm.FormCreate.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, Grids, DB, DBGrids, ADODB, StdCtrls;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    ExitMenu: TMenuItem;
    HelpMenu: TMenuItem;
    AboutMenu: TMenuItem;
    CustomGrid: TDBGrid;
    Connection1: TADOConnection;
    ADOTable1: TADOTable;
    DataSource1: TDataSource;
    Label1: TLabel;
    Label2: TLabel;
    DefaultGrid: TDBGrid;
    procedure ExitMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);

  procedure SetCaption(index: Integer; const caption: String);
  begin
    // Sets the Caption properties of the custom grid
    CustomGrid.Columns[index].Title.Caption := caption;

    // Sets the Caption properties of the default grid
    DefaultGrid.Columns[index].Title.Caption := caption;
  end;

resourcestring
  SId = 'Id';
  SName = 'Name';
  SPopulation = 'Population';
  SCapital = 'Capital';
  SDescription = 'Description';
begin
  // Assign the DBGrid.Columns[].Title.Caption properties.
  SetCaption(0, SId);
  SetCaption(1, SName);
  SetCaption(2, SPopulation);
  SetCaption(3, SCapital);
  SetCaption(4, SDescription);
end;

procedure TMainForm.ExitMenuClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AboutMenuClick(Sender: TObject);
resourcestring
  SAbout = 'This application demonstrates how to localize database content.';
begin
  ShowMessage(SAbout);
end;

end.
