unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XPMan, NetModule_TLB;

type
  TShowFormProc = function (language: PChar): Integer; stdcall;
  TSetLanguageProc = function (value: PChar): Integer; stdcall;

  TMainForm = class(TForm)
    LanguageButton: TButton;
    LanguageLabel: TLabel;
    VclButton: TButton;
    VclLabel: TLabel;
    NetButton: TButton;
    NetLabel: TLabel;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure LanguageButtonClick(Sender: TObject);
    procedure VclButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure NetButtonClick(Sender: TObject);

  private
    FVclHandle: THandle;
    FVclShowFormProc: TShowFormProc;
    FVclSetLanguage: TSetLanguageProc;
    FNetModule: ILaModule;

    function GetLanguageAsPChar: PChar;

  public
    property LanguageAsPChar: PChar  read GetLanguageAsPChar;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  ComObj,
  LaDialog,
  LaResource;

function TMainForm.GetLanguageAsPChar: PChar;
begin
  Result := PChar(GetActiveResourceLocale('EN'));
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Load VCL DLL and get functions
  FVclHandle := LoadLibrary('VclModule.dll');

  if FVclHandle <> 0 then
  begin
    FVclShowFormProc := GetProcAddress(FVclHandle, 'ShowForm');
    FVclSetLanguage := GetProcAddress(FVclHandle, 'SetLanguage');
  end;

  // Get the interface of .NET module
  try
    FNetModule := CreateComObject(CLASS_LaModule) as ILaModule;
  except
    FNetModule := nil;
  end;

  // Disable buttons that are not available
  VclButton.Enabled := FVclHandle <> 0;
  NetButton.Enabled := FNetModule <> nil;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if FVclHandle <> 0 then
    FreeLibrary(FVclHandle);
end;

procedure TMainForm.LanguageButtonClick(Sender: TObject);
begin
  // Let the user to choose a new language and after it has been changed
  // call the SetLanguage functions of DLLs (if loaded) to change the language
  // of DLL to match the application.
  if SelectResourceLocale('EN') then
  begin
    if Assigned(FVclSetLanguage) then
      FVclSetLanguage(LanguageAsPChar);

    if FNetModule <> nil then
      FNetModule.SetLanguage(LanguageAsPChar);
  end;
end;

procedure TMainForm.VclButtonClick(Sender: TObject);
begin
  if Assigned(FVclShowFormProc) then
    FVclShowFormProc(LanguageAsPChar);
end;

procedure TMainForm.NetButtonClick(Sender: TObject);
begin
  if FNetModule <> nil then
    FNetModule.ShowForm(LanguageAsPChar);
end;

end.
