{
  This sample shows how to use the functions in the LaResource.pas to get
  available languages and how to show them in a radio group and finally
  how to turn on a new language after the user has selected it.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    RadioGroup1: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  LaCommon,
  LaResource,
  LaTranslator;

procedure TForm1.FormCreate(Sender: TObject);

  procedure Process(locale: Integer);
  var
    str: String;
  begin
    str := GetWindowsLocaleName(locale, False);
    RadioGroup1.Items.AddObject(str, TObject(locale));
  end;

var
  i, locale: Integer;
  code: String;
  languages: TStringList;
begin
  // Populate the radio group to contains the possible languages

  // Original language is English
  Process(LANG_ENGLISH);

  // Add the available languages.
  // Get only those that are compatible to the current code page.
  languages := TStringList.Create;
  try
    GetAvailableResources(languages, '', True);

    for i := 0 to languages.Count - 1 do
    begin
      code := languages[i];
      locale := ExtensionToLocale(code);
      Process(locale);

      if code = LoadedResourceLocale then
        RadioGroup1.ItemIndex := RadioGroup1.Items.Count - 1;
    end;

    if RadioGroup1.ItemIndex = -1 then
      RadioGroup1.ItemIndex := 0;
  finally
    languages.Free;
  end;
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
var
  i, locale: Integer;
  code: String;
begin
  locale := Integer(RadioGroup1.Items.Objects[RadioGroup1.ItemIndex]);

  // Original language is English so use empty language code with English
  if locale = LANG_ENGLISH then
    code := ''
  else
    code := LocaleToExtension(locale);

  SetNewResourceFile(code);

  // Translate the radio buttons
  for i := 0 to RadioGroup1.Items.Count - 1 do
    RadioGroup1.Items[i] := GetWindowsLocaleName(
      Integer(RadioGroup1.Items.Objects[i]),
      False);
end;

initialization
  // Translate only string properties when performing runtime language switch
  LaEnabledProperties := STRING_TYPES_C;
end.
