{
  Copyrights 2005-2008 Sisulizer Ltd.

  This sample shows how to
  1) Disable runtime language switch of all properties except strings
  2) How to perform runtime language switch of a resource string array
  3) How to replace resource string arrays because they are difficuly during
     runtime language switch

  Read comments in the code.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    ListBox1: TListBox;
    ComboBox1: TComboBox;
    Label1: TLabel;
    RadioGroup1: TRadioGroup;
    RadioGroup2: TRadioGroup;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  TypInfo,
  LaDialog,
  LaTranslator;

resourcestring
  SCar = 'Car';
  STrain = 'Train';
  SBike = 'Bike';

{$J+} // This is needed to make RES_ARRAY writeable

const
  RES_ARRAY: array[0..2] of String =
  (
    SCar,
    STrain,
    SBike
  );

// Above code is used very often in VCL applications and in fact in VCL itself
// There is nothing wrong in that but makes runtime language switch difficult.
// The problem is that the array is populated on run time using the initial
// resource string value. If you later change the language the array will still
// have values in old language.
// You have two choices:
// 1) Translate resource string array manually after languages change
// 2) Recommened: Do not use resource array but replace them with function
//    that takes index parameter and returns resource string. See below.

const
  RES_VALUE_COUNT_C = 3;

function GetResValue(i: Integer): WideString;
begin
  case i of
    0: Result := SCar;
    1: Result := STrain;
    2: Result := SBike;
  else
    raise Exception.Create('List out of index bound');
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  // Change properties on run time
  ClientWidth := 600;
  ClientHeight := 250;

  // This uses options #1 where strings comes from resource string array.
  for i := Low(RES_ARRAY) to High(RES_ARRAY) do
    RadioGroup1.Items.Add(RES_ARRAY[i]);

  RadioGroup1.ItemIndex := 0;

  // This uses options #2 where strings comes from resource string fucntion
  for i := 0 to RES_VALUE_COUNT_C - 1 do
    RadioGroup2.Items.Add(GetResValue(i));

  RadioGroup2.ItemIndex := 1;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i: Integer;
begin
  // Performs runtime language switch
  if SelectResourceLocale('EN') then
  begin
    // A new resource DLL has been loaded and TLaTranslator has automatically
    // translated all component to the default values using the values from
    // resource DLL. However TLaTranslator did not translate changes we did in
    // FormCreate event. We have to manualy translate radio groups.

    // Translate RES_ARRAY. Assing each value by its resource string.
    // Now resource strings contains new values.
    // Finally translate the radio group 1.
    RES_ARRAY[0] := SCar;
    RES_ARRAY[1] := STrain;
    RES_ARRAY[2] := SBike;

    for i := Low(RES_ARRAY) to High(RES_ARRAY) do
      RadioGroup1.Items[i] := RES_ARRAY[i];

    // No need to translate resource string array because we do not use it
    // but a function. Just translate the radio group 2.
    for i := 0 to RES_VALUE_COUNT_C - 1 do
      RadioGroup2.Items[i] := GetResValue(i);
  end;
end;

initialization
  // This makes TLaTranslator to translate only properties that contain string data
  LaEnabledProperties := STRING_TYPES_C;
end.
