{
  Copyrights 2005-2008 Sisulizer Ltd.

  This application shows how to have two languages in a single application.
  The application contains two languages:
  - Main language. Most of the forms and items are in this language.
  - Client language. Some forms and items are in this language.

  Application uses embedded resource DLL to perform language switch.
  When application start it will use only single language and this language is
  the default language choosen by VCL. On runtime the user can change the main
  and client languages using the combo boxes on the top of the main menu.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    MainLabel: TLabel;
    ClientLabel: TLabel;
    MainCombo: TComboBox;
    ClientCombo: TComboBox;
    PageControl1: TPageControl;
    BankSheet: TTabSheet;
    ClientSheet: TTabSheet;
    ListBox1: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    NameEdit: TEdit;
    Label3: TLabel;
    AgeEdit: TEdit;
    UpDown1: TUpDown;
    BankButton: TButton;
    ClientButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure MainComboChange(Sender: TObject);
    procedure ClientComboChange(Sender: TObject);
    procedure BankButtonClick(Sender: TObject);
    procedure ClientButtonClick(Sender: TObject);

  private
    function GetLanguage(i: Integer): String;
    function GetLanguageCount: Integer;
    function GetMainLanguage: String;
    function GetClientLanguage: String;

  public
    property LanguageCount: Integer read GetLanguageCount;
    property Languages[i: Integer]: String read GetLanguage;
    property MainLanguage: String read GetMainLanguage;
    property ClientLanguage: String read GetClientLanguage;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  TypInfo,
  LaResource,
  LaTranslator,
  BankOptionsDlg,
  ClientOptionsDlg;

function TForm1.GetLanguageCount: Integer;
begin
  Result := MainCombo.Items.Count;
end;

function TForm1.GetLanguage(i: Integer): String;
begin
  Result := TLanguageListItem(MainCombo.Items.Objects[i]).Code;
end;

function TForm1.GetMainLanguage: String;
begin
  if MainCombo.ItemIndex > 0 then
    Result := Languages[MainCombo.ItemIndex]
  else
    Result := '';
end;

function TForm1.GetClientLanguage: String;
begin
  if ClientCombo.ItemIndex > 0 then
    Result := Languages[ClientCombo.ItemIndex]
  else
    Result := '';
end;

procedure TForm1.FormCreate(Sender: TObject);

  procedure AddString(list: TComboBox; const name: String; const code: String);
  begin
    list.Items.AddObject(name, TLanguageListItem.Create(code, name));
  end;

  procedure Process(const code: String);
  var
    locale: Integer;
  begin
    locale := ExtensionToLocale(code);

    AddString(MainCombo, GetWindowsLocaleName(locale, True), code);
    AddString(ClientCombo, GetWindowsLocaleName(locale, True), code);
  end;

var
  i: Integer;
  availableLanguages: TStringList;
begin
  // Populates the language combo boxes
  availableLanguages := TStringList.Create;
  try
    GetAvailableResources(availableLanguages);

    // Original language of this application is English
    Process('EN');

    // Add the languages of the resource DLLs that were found
    for i := 0 to availableLanguages.Count - 1 do
      Process(availableLanguages[i]);
  finally
    availableLanguages.Free;
  end;

  // Set the default language in the combo boxes
  for i := 0 to LanguageCount - 1 do
  begin
    if ((LoadedResourceLocale <> '') and (Languages[i] = LoadedResourceLocale)) or
      ((LoadedResourceLocale = '') and (Languages[i] = 'EN')) then
    begin
      MainCombo.ItemIndex := i;
      ClientCombo.ItemIndex := i;
      Break;
    end;
  end;

  if (MainCombo.ItemIndex = -1) and (MainCombo.Items.Count > 0) then
  begin
    MainCombo.ItemIndex := 0;
    ClientCombo.ItemIndex := 0;
  end;

  // Initialize the form
  PageControl1.ActivePageIndex := 0;
end;

// This funtion returns true if the given object belongs to client item.
// host is form or frame where the object belongs
// obj  is the object (component or its part) that is currently being translated
function IsClientItem(
  host: TComponent;
  obj: TObject): Boolean;
begin
  // ClientOptionsDialog and ClientSheet are client items.
  Result :=
    (host = ClientOptionsDialog) or
    (host = Form1) and ((obj = Form1.ClientSheet) or (obj is TControl) and (TControl(obj).Parent = Form1.ClientSheet));
end;

// Translation event that cancels translation of all client items
procedure TranslateMainItems(
  host: TComponent;
  obj: TObject;
  propertyInfo: PPropInfo;
  const currentValue: Variant;
  var newValue: Variant;
  var cancel: Boolean);
begin
  if IsClientItem(host, obj) then
    cancel := True;
end;

// Translation event that cancels translation of all items that are not client items
procedure TranslateClientItems(
  host: TComponent;
  obj: TObject;
  propertyInfo: PPropInfo;
  const currentValue: Variant;
  var newValue: Variant;
  var cancel: Boolean);
begin
  if not IsClientItem(host, obj) then
    cancel := True;
end;

procedure TForm1.MainComboChange(Sender: TObject);
begin
  // Change main language.
  // Uses LaBeforeTranslate event to control what items to translate.
  LaBeforeTranslate := TranslateMainItems;
  SetNewResourceFile(MainLanguage);

  // If your code requires settings on run time some properties that affect
  // user interface perform the code hear after language switch.
end;

procedure TForm1.ClientComboChange(Sender: TObject);
begin
  // Change client language
  // Uses LaBeforeTranslate event to control what items to translate.
  LaBeforeTranslate := TranslateClientItems;
  SetNewResourceFile(ClientLanguage);

  // If your code requires settings on run time some properties that affect
  // user interface perform the code hear after language switch.

  // Finally reoad the main language resource so all forms that are created
  // from now on will appear in main language. Also all resourcestring values
  // are now back in main language.
  LoadNewResourceFile(MainLanguage);
end;

procedure TForm1.BankButtonClick(Sender: TObject);
begin
  BankOptionsDialog.ShowModal;
end;

procedure TForm1.ClientButtonClick(Sender: TObject);
begin
  ClientOptionsDialog.ShowModal;
end;

initialization
  // This makes Sisulizer to translate only string properties after language
  // change. This eliminates flickering when position won't be changed back to
  // the default values.
  LaEnabledProperties := STRING_TYPES_C;

  // This function extracts embedded resource DLLs from custom resource
  // to resource DLL files.
  ExtractResourceFiles;
end.
