{
  Copyrights 2005-2011 Sisulizer

  Unit that translates images.
  This compiles with Delphi 5 and later
}

{$I LaDef.inc}

unit LaImageModule;

interface

uses
  Classes,
  LaTranslator;

type
  TLaImageModule = class(TLaModule)
  public
    function CanTranslate(obj: TObject): Boolean; override;

    procedure Translate(
      component: TComponent;
      obj: TObject;
      const name: String;
      value: Variant;
      index: Integer); override;
  end;

implementation

uses
  SysUtils,
{$IFDEF VCL6}
  Variants,
{$ENDIF}
  Graphics;

function TLaImageModule.CanTranslate(obj: TObject): Boolean;
begin
  Result := obj is TPicture;
end;

procedure TLaImageModule.Translate(
  component: TComponent;
  obj: TObject;
  const name: String;
  value: Variant;
  index: Integer);
var
  stream: TMemoryStream;

  procedure ProcessSize;
  var
    size: Longint;
  begin
    stream.Read(size, SizeOf(size));
  end;

var
  nameLen: Byte;
  pictureClassName: String;
  ansi: AnsiString;
  picture: TPicture;
  currentGraphic, newGraphic: TGraphic;
begin
  if VarType(value) <> varString then
    Exit;

  picture := obj as TPicture;
  currentGraphic := picture.Graphic;
  ansi := AnsiString(value);

  stream := nil;
  newGraphic := nil;
  try
    stream := TMemoryStream.Create;
    stream.Write(ansi[1], Length(ansi));
    stream.Seek(0, soFromBeginning);

    stream.Read(nameLen, 1);
    SetLength(ansi, nameLen);
    stream.Read(ansi[1], nameLen);
    pictureClassName := String(ansi);

    if not SameText(currentGraphic.ClassName, pictureClassName) then
      Exit;

    newGraphic := TGraphicClass(currentGraphic.ClassType).Create;

    if SameText(pictureClassName, 'TJpegImage') or SameText(pictureClassName, 'TBitmap') then
      ProcessSize;

    newGraphic.LoadFromStream(stream);
    picture.Graphic := newGraphic;
  finally
    newGraphic.Free;
    stream.Free;
  end;
end;

initialization
  LaModules.Register(TLaImageModule);
end.
