{
  Copyrights 2005-2011 Sisulizer

  Unit that translates TListView's items.
  This compiles with Delphi 5 and later
}

{$I LaDef.inc}

unit LaListViewModule;

interface

uses
  Classes,
  LaTranslator;

type
  TLaListViewModule = class(TLaModule)
  public
    function CanTranslate(obj: TObject): Boolean; override;

    procedure Translate(
      component: TComponent;
      obj: TObject;
      const name: String;
      value: Variant;
      index: Integer); override;
  end;

implementation

uses
  ComCtrls,
  LaCommon;

type
  TItemInfo = packed record
    ImageIndex: Integer;
    StateIndex: Integer;
    OverlayIndex: Integer;
    SubItemCount: Integer;
    Data: Pointer;
  end;

  TItemInfo2 = packed record
    ImageIndex: Integer;
    StateIndex: Integer;
    OverlayIndex: Integer;
    SubItemCount: Integer;
    GroupID: Integer;
    Data: Pointer;
  end;

function TLaListViewModule.CanTranslate(obj: TObject): Boolean;
begin
  Result := obj is TListItems;
end;

procedure TLaListViewModule.Translate(
  component: TComponent;
  obj: TObject;
  const name: String;
  value: Variant;
  index: Integer);
const
  DELPHI_2009_VERSION = $03;  // 32-bit struct size version 2
  DELPHI_XE_VERSION = $05;    // 32-bit struct size version 3
var
  stream: TLaStream;
  items: TListItems;

  procedure ProcessOld;
  var
    i, j, count: Integer;
    item: TListItem;
    info: TItemInfo;
  begin
    stream.ReadInteger; // size
    count := stream.ReadInteger;

    for i := 0 to count - 1 do
    begin
      item := items[i];

      stream.Read(info, Sizeof(info));
      item.ImageIndex := info.ImageIndex;
      item.StateIndex := info.StateIndex;
      item.OverlayIndex := info.OverlayIndex;
      item.Data := info.Data;
      item.Caption := AnsiToUnicode(stream.ReadShortString);

      for j := 0 to info.SubItemCount - 1 do
        item.SubItems[j] := AnsiToUnicode(stream.ReadShortString);
    end;
  end;

{$IFDEF VCL10}
  procedure ProcessNew;
  var
    item: TListItem;

    procedure Process;
    var
      i: Integer;
      info: TItemInfo;
    begin
      stream.Read(info, Sizeof(info));
      item.ImageIndex := info.ImageIndex;
      item.StateIndex := info.StateIndex;
      item.OverlayIndex := info.OverlayIndex;
      item.Data := info.Data;
      item.Caption := stream.ReadShortUnicodeString;

      for i := 0 to info.SubItemCount - 1 do
        item.SubItems[i] := stream.ReadShortUnicodeString;
    end;

{$IFDEF UNICODE_VCL}
    procedure Process2(version: Byte);
    var
      i: Integer;
      info: TItemInfo2;
    begin
      stream.Read(info, Sizeof(info));
      item.ImageIndex := info.ImageIndex;
      item.StateIndex := info.StateIndex;
      item.OverlayIndex := info.OverlayIndex;
      item.Data := info.Data;
      item.Caption := stream.ReadShortUnicodeString;

      for i := 0 to info.SubItemCount - 1 do
      begin
        item.SubItems[i] := stream.ReadShortUnicodeString;

        if version >= DELPHI_XE_VERSION then
          stream.ReadPointer;
      end;
    end;
  {$ENDIF}

  var
    i, count: Integer;
{$IFDEF UNICODE_VCL}
    version: Byte;
{$ENDIF}
  begin
    {$IFDEF UNICODE_VCL}version := {$ENDIF}stream.ReadByte; // version
    stream.ReadInteger; // size
    count := stream.ReadInteger;

    for i := 0 to count - 1 do
    begin
      item := items[i];

{$IFDEF UNICODE_VCL}
      if version >= DELPHI_2009_VERSION then
        Process2(version)
      else
{$ENDIF}
        Process;
    end;
  end;
{$ENDIF}

begin
  items := obj as TListItems;

  stream := TLaStream.Create(AnsiString(value));
  try
    if name = 'Items.Data' then
      ProcessOld
{$IFDEF VCL10}
    else
      ProcessNew
{$ENDIF}
  finally
    stream.Free;
  end;
end;

initialization
  LaModules.Register(TLaListViewModule);
end.
