{
  Copyrights 2005-2011 Sisulizer

  Unit that translates TTreeView's nodes.
  This compiles with Delphi 5 and later
}

{$I LaDef.inc}

unit LaTreeViewModule;

interface

uses
  Classes,
  LaTranslator;

type
  TLaTreeViewModule = class(TLaModule)
  public
    function CanTranslate(obj: TObject): Boolean; override;

    procedure Translate(
      component: TComponent;
      obj: TObject;
      const name: String;
      value: Variant;
      index: Integer); override;
  end;

implementation

uses
  ComCtrls,
{$IFDEF VER140}
  Variants,
{$ENDIF}
  LaCommon;

function TLaTreeViewModule.CanTranslate(obj: TObject): Boolean;
begin
  Result := obj is TTreeNodes;
end;

procedure TLaTreeViewModule.Translate(
  component: TComponent;
  obj: TObject;
  const name: String;
  value: Variant;
  index: Integer);
const
  DELPHI_2009_VERSION = $03;  // 32-bit struct size version 2
var
  stream: TLaStream;

  procedure ProcessOld(node: TTreeNode);
  var
    i, size: Integer;
    info: TNodeInfo;
  begin
    size := stream.ReadInteger;
    stream.Read(info, size);

    node.ImageIndex := info.ImageIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.OverlayIndex := info.OverlayIndex;
    node.Text := AnsiToUnicode(info.Text);

    for i := 0 to info.Count - 1 do
      ProcessOld(node[i]);
  end;

{$IFDEF VCL10}
  procedure ProcessNew(node: TTreeNode);
  var
    i: Integer;
    info: TNodeDataInfo;
    str: UnicodeString;
  begin
    stream.ReadInteger; // size
    stream.Read(info, SizeOf(info));

    SetLength(str, info.TextLen);

    if info.TextLen > 0 then
      stream.Read(str[1], 2*info.TextLen);

    node.ImageIndex := info.ImageIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.OverlayIndex := info.OverlayIndex;
    node.Text := str;

    for i := 0 to info.Count - 1 do
      ProcessNew(node[i]);
  end;
{$ENDIF}

{$IFDEF UNICODE_VCL}
  procedure ProcessNew2(node: TTreeNode);
  var
    i: Integer;
{$IFDEF WIN64}
    info: TNodeDataInfo2x86;
{$ELSE}
    info: TNodeDataInfo2;
{$ENDIF}
    str: UnicodeString;
  begin
    stream.ReadInteger; // size
    stream.Read(info, SizeOf(info));

    SetLength(str, info.TextLen);

    if info.TextLen > 0 then
      stream.Read(str[1], 2*info.TextLen);

    node.ImageIndex := info.ImageIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.SelectedIndex := info.SelectedIndex;
    node.OverlayIndex := info.OverlayIndex;
    node.Text := str;

    for i := 0 to info.Count - 1 do
      ProcessNew2(node[i]);
  end;
{$ENDIF}

var
  i, count: Integer;
  node: TTreeNode;
  nodes: TTreeNodes;
{$IFDEF UNICODE_VCL}
  version: Byte;
{$ENDIF}
begin
  nodes := obj as TTreeNodes;

  stream := TLaStream.Create(RawByteString(value));
  try
    if name = 'Items.Data' then
    begin
      count := stream.ReadInteger;
      node := nodes.GetFirstNode;

      for i := 0 to count - 1 do
      begin
        ProcessOld(node);
        node := node.getNextSibling;
      end;
    end
{$IFDEF VCL10}
    else
    begin
      {$IFDEF UNICODE_VCL}version := {$ENDIF}stream.ReadByte;
      count := stream.ReadInteger;
      node := nodes.GetFirstNode;

      for i := 0 to count - 1 do
      begin
{$IFDEF UNICODE_VCL}
        if version >= DELPHI_2009_VERSION then
          ProcessNew2(node)
        else
{$ENDIF}
          ProcessNew(node);

        node := node.getNextSibling;
      end;
    end;
{$ENDIF}
  finally
    stream.Free;
  end;
end;

initialization
  LaModules.Register(TLaTreeViewModule);
end.
