unit LaVirtualTreeModule;

interface

uses
  Classes,
  LaTranslator;

type
  TLaVirtualTreeModule = class(TLaModule)
  public
    function CanTranslate(obj: TObject): Boolean; override;

    procedure Translate(
      component: TComponent;
      obj: TObject;
      const name: String;
      value: Variant;
      index: Integer); override;

    function GetActualObject(obj: TObject; const propName: String): TObject; override;
    function GetActualName(obj: TObject; const propName: String): String; override;
  end;

implementation

uses
  VirtualTrees;

function TLaVirtualTreeModule.CanTranslate(obj: TObject): Boolean;
begin
  Result := obj is TVirtualTreeColumn;
end;

procedure TLaVirtualTreeModule.Translate(
  component: TComponent;
  obj: TObject;
  const name: String;
  value: Variant;
  index: Integer);
var
  item: TVirtualTreeColumn;
begin
  if obj is TVirtualTreeColumn then
  begin
    item := TVirtualTreeColumn(obj);

    if name = 'WideText' then
      item.Text := value
    else if name = 'WideHint' then
      item.Hint := value
  end
end;

function TLaVirtualTreeModule.GetActualObject(
  obj: TObject;
  const propName: String): TObject;
begin
  if (obj is TVirtualStringTree) and (propName = 'Columns') then
    Result := TVirtualStringTree(obj).Header
  else
    Result := nil;
end;

function TLaVirtualTreeModule.GetActualName(
  obj: TObject;
  const propName: String): String;
begin
  if (obj is TVirtualTreeColumn) and (propName = 'WideText') then
    Result := 'Text'
  else if (obj is TVirtualTreeColumn) and (propName = 'WideHint') then
    Result := 'Hint'
  else
    Result := '';
end;

initialization
  LaModules.Register(TLaVirtualTreeModule);
end.
