﻿{
  Copyrights 2005-2011 Sisulizer

  This sample demonstrates how to keep internal data of an application in
  Unicode and how to read and write external data to various formats.
  The application itself is very simple. It only contains list of strings.
  The application can read and write either xml, text or binary files.
  The application can read and write data in UTF-8, UTF-16 and using all the
  code pages that are supported by the computer.

  As you will see reading and writing from/to an XML file is the easiest and
  simplest way to store data. We recommend to keep your data in XML rather than
  in text or binary formats.

  Using Microsoft XML parser is very compilcated and it is a very limited and
  slow. This is why we at Sisulizer has written our own XML parser that is 100%
  native Delphi. It is very simple to use, fast and support all character
  endodings. However it is not publicly available. This sample uses the class
  but the source code is not included. Instead we deliver precompiled Delphi
  2009 units.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ExtCtrls;

type
  TFileFormat = (ffXml, ffText, ffBinary);

  TForm1 = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    NewMenu: TMenuItem;
    OpenMenu: TMenuItem;
    SaveMenu: TMenuItem;
    SaveAsMenu: TMenuItem;
    ExitMenu: TMenuItem;
    PopulateMenu: TMenuItem;
    EnglishMenu: TMenuItem;
    FinnishFinnish: TMenuItem;
    FrenchMenu: TMenuItem;
    GermanMenu: TMenuItem;
    JapaneseMenu: TMenuItem;
    SwedishMenu: TMenuItem;
    HelpMenu: TMenuItem;
    AboutMenu: TMenuItem;
    ItemsLabel: TLabel;
    ItemsMemo: TMemo;
    FormatGroup: TRadioGroup;
    EncodingLabel: TLabel;
    EncodingCombo: TComboBox;
    PolishMenu: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure NewMenuClick(Sender: TObject);
    procedure OpenMenuClick(Sender: TObject);
    procedure SaveMenuClick(Sender: TObject);
    procedure SaveAsMenuClick(Sender: TObject);
    procedure ExitMenuClick(Sender: TObject);
    procedure EnglishMenuClick(Sender: TObject);
    procedure FinnishFinnishClick(Sender: TObject);
    procedure FrenchMenuClick(Sender: TObject);
    procedure GermanMenuClick(Sender: TObject);
    procedure JapaneseMenuClick(Sender: TObject);
    procedure SwedishMenuClick(Sender: TObject);
    procedure AboutMenuClick(Sender: TObject);
    procedure ItemsMemoChange(Sender: TObject);
    procedure PolishMenuClick(Sender: TObject);

  private
    FFileName: String;

    function GetCodePage: Integer;
    function GetCount: Integer;
    function GetFileFormat: TFileFormat;
    function GetItem(i: Integer): String;

    procedure SetCodePage(value: Integer);
    procedure SetFileFormat(value: TFileFormat);

    function GetCodePageAt(index: Integer): Integer;

    procedure Add(const value: String);
    procedure Clear;
    procedure Populate(one, two, three, four, five: String; codePage: Integer);

    procedure Save(const fileName: String);
    procedure Open(const fileName: String);

    procedure UpdateComponents;

  public
    property CodePage: Integer read GetCodePage write SetCodePage;
    property Count: Integer read GetCount;
    property FileFormat: TFileFormat read GetFileFormat write SetFileFormat;
    property Items[i: Integer]: String read GetItem;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  LaCommon,
  // This units are delivered as precompiled Delphi 2009 units.
  // The units are used internally in Sisulizer and are not publicly available.
  LaScript,
  LaXml;

type
  // A stucture that contains the header bytes of application binary file format (.dat)
  THeader = array[0..3] of Byte;

const
  // UTF8 and UTF16 do not uses code pages so application assing zero or
  // negative code page numbers for these
  UTF8_C = 0;
  UTF16LE_C = -1;
  UTF16BE_C = -2;

  // Some Windows code pages
  JAPANESE_CP_C = 932;
  EASTERN_CP_C = 1250;
  WESTERN_CP_C = 1252;

  // XML, text and binary file extension
  XML_C = 'xml';
  TEXT_C = 'txt';
  BINARY_C = 'dat';

  // Header bytes of the binary file format (.dat)
  HEADER_C: THeader = ($10, $11, $12, $13);

  // UTF-8 and UTF-16 headers/byte order marks
  UTF8_BOM_C: array[0..2] of Byte = ($EF, $BB, $BF);
  UTF16_BOM_C: WideChar = WideChar($FEFF);

  // New line characters
  ANSI_NEW_LINE_C: array[0..1] of AnsiChar = #13#10;
  WIDE_NEW_LINE_C: array[0..1] of WideChar = #13#10;

  XML_ITEMS_C = 'items';
  XML_ITEM_C = 'item';

var
  enumCombo: TComboBox;

resourcestring
  SXmlFiles = 'XML files';
  STextFiles = 'Text files';
  SBinaryFiles = 'Binary files';

// Converts the byte order of the given Unicode character
function ChangeWideCharByteOrder(value: WideChar): WideChar;
begin
  Result :=
    WideChar(((Word(value) and $00FF) shl 8) or
    ((Word(value) and $FF00) shr 8));
end;

// Returns true of the header datas equal
function HeadersEqual(header1, header2: THeader): Boolean;
var
  i: Integer;
begin
  for i := Low(THeader) to High(THeader) do
    if header1[i] <> header2[i]then
    begin
      Result := False;
      Exit;
    end;

  Result := True;
end;

// Updates Caption property and menu items
procedure TForm1.UpdateComponents;
resourcestring
  SFileSample = 'File Sample';
  SUntitled = 'Untitled';

  function GetName: String;
  begin
    if FFileName <> '' then
      Result := ExtractFileName(FFileName)
    else
      Result := SUntitled;
  end;

begin
  Caption := SFileSample + ' - ' + GetName;
  SaveMenu.Enabled := Count > 0;
  SaveAsMenu.Enabled := Count > 0;
end;

function TForm1.GetCodePageAt(index: Integer): Integer;
begin
  Result := Integer(EncodingCombo.Items.Objects[index]);
end;

function TForm1.GetCodePage: Integer;
begin
  Result := GetCodePageAt(EncodingCombo.ItemIndex);
end;

procedure TForm1.SetCodePage(value: Integer);
resourcestring
  SUnknownCodePage = 'Unknown code page: %d';
var
  i: Integer;
begin
  for i := 0 to EncodingCombo.Items.Count - 1 do
  begin
    if GetCodePageAt(i) = value then
    begin
      EncodingCombo.ItemIndex := i;
      Exit;
    end;
  end;

  EncodingCombo.ItemIndex := -1;
  raise Exception.CreateFmt(SUnknownCodePage, [value]);
end;

function TForm1.GetFileFormat: TFileFormat;
begin
  Result := TFileFormat(FormatGroup.ItemIndex);
end;

procedure TForm1.SetFileFormat(value: TFileFormat);
begin
  FormatGroup.ItemIndex := Ord(value);
end;

function TForm1.GetCount: Integer;
begin
  Result := ItemsMemo.Lines.Count;
end;

function TForm1.GetItem(i: Integer): String;
begin
  Result := ItemsMemo.Lines[i];
end;

procedure TForm1.Add(const value: String);
begin
  ItemsMemo.Lines.Add(value);
end;

procedure TForm1.Clear;
begin
  ItemsMemo.Lines.Clear;
end;

// Writes data to a file
procedure TForm1.Save(const fileName: String);

  procedure SaveXml;
  var
    i: Integer;
    xml: TLaXmlFile;
  begin
    // Write data to an XML file.
    xml := TLaXmlFile.Create;
    try
      // Set encoding
      if CodePage = UTF8_C then
        xml.Encoding := ENCODING_UTF8_C
      else if CodePage = UTF16LE_C then
        xml.Encoding := ENCODING_UTF16LE_C
      else if CodePage = UTF16BE_C then
        xml.Encoding := ENCODING_UTF16BE_C
      else
        xml.CodePage := CodePage;

      // Write header
      xml.Rewrite(fileName);

      // Start main element
      xml.WriteStartTag(XML_ITEMS_C);

      // Write sub elements that contain the string values
      for i := 0 to Count - 1 do
        xml.WriteString(XML_ITEM_C, Items[i]);

      // Close main element
      xml.WriteEndTag;
    finally
      xml.Free;
    end;
  end;

  procedure SaveText;
  var
    i, j, len: Integer;
    c: WideChar;
    str: String;
    utf8: Utf8String;
    ansi: AnsiString;
    stream: TFileStream;
  begin
    // Write data to a text file
    stream := TFileStream.Create(fileName, fmCreate);
    try
      // Write the header.
      // UTF-8 and UTF-16 uses UTF BOMs.
      // Ansi files contain the code page in the first line
      if CodePage = UTF8_C then
        stream.Write(UTF8_BOM_C[0], Sizeof(UTF8_BOM_C))
      else if CodePage = UTF16LE_C then
        stream.Write(UTF16_BOM_C, Sizeof(UTF16_BOM_C))
      else if CodePage = UTF16BE_C then
      begin
        c := ChangeWideCharByteOrder(UTF16_BOM_C);
        stream.Write(c, Sizeof(c));
      end
      else
      begin
        ansi := UnicodeToAnsi(IntToStr(CodePage)) + ANSI_NEW_LINE_C;
        stream.Write(ansi[1], Length(ansi));
      end;

      for i := 0 to Count - 1 do
      begin
        str := Items[i];

        if CodePage = UTF8_C then
        begin
          // Convert string to UTF-8, add new line characters and write the line.
          utf8 := Utf8Encode(str) + ANSI_NEW_LINE_C;
          stream.Write(utf8[1], Length(utf8));
        end
        else if CodePage = UTF16LE_C then
        begin
          // String are internally in UTF-16 LE. No need to convert.
          // Just write add the new line characters and write the line.
          str := str + WIDE_NEW_LINE_C;
          stream.Write(str[1], 2*Length(str));
        end
        else if CodePage = UTF16BE_C then
        begin
          // As above but converts the byte of order of the string before writing.
          str := str + WIDE_NEW_LINE_C;
          len := Length(str);

          for j := 1 to len do
            str[j] := ChangeWideCharByteOrder(str[j]);

          stream.Write(str[1], 2*len);
        end
        else
        begin
          // Convert string to ANSI, add new line characters and write the line.
          ansi := UnicodeToAnsi(str, CodePage) + ANSI_NEW_LINE_C;
          stream.Write(ansi[1], Length(ansi));
        end
      end;
    finally
      stream.Free;
    end;
  end;

  procedure SaveBinary;
  var
    stream: TFileStream;

    procedure WriteWord(value: Word);
    begin
      stream.Write(value, Sizeof(value));
    end;

  var
    i, j, len: Integer;
    str: String;
    utf8: Utf8String;
    ansi: AnsiString;
  begin
    stream := TFileStream.Create(fileName, fmCreate);
    try
      // Write header. It is 8 bytes and contains:
      // - file tag   4 bytes
      // - code page  2 bytes
      // - item count 2 bytes
      stream.Write(HEADER_C[0], Sizeof(HEADER_C));
      WriteWord(CodePage);
      WriteWord(Count);

      // Write items
      for i := 0 to Count - 1 do
      begin
        str := Items[i];

        if CodePage = UTF8_C then
        begin
          // Convert string to UTF-8 and write the bytes.
          utf8 := Utf8Encode(str);
          len := Length(utf8);
          WriteWord(len);
          stream.Write(utf8[1], len);
        end
        else if CodePage = UTF16LE_C then
        begin
          // String are internally in UTF-16 LE. No need to convert. Just write the bytes.
          len := Length(str);
          WriteWord(len);
          stream.Write(str[1], 2*len);
        end
        else if CodePage = UTF16BE_C then
        begin
          // As above but converts the byte of order of the string before writing.
          len := Length(str);
          WriteWord(len);

          for j := 1 to len do
            str[j] := ChangeWideCharByteOrder(str[j]);

          stream.Write(str[1], 2*len);
        end
        else
        begin
          // Convert string to ANSI and write the bytes.
          ansi := UnicodeToAnsi(str, CodePage);
          len := Length(ansi);
          WriteWord(len);
          stream.Write(ansi[1], len);
        end
      end;
    finally
      stream.Free;
    end;
  end;

begin
  FFileName := fileName;

  case FileFormat of
    ffXml: SaveXml;
    ffText: SaveText;
    ffBinary: SaveBinary;
  end;

  UpdateComponents;
end;

// Reads data from a file
procedure TForm1.Open(const fileName: String);

  procedure OpenXml;
  var
    i: Integer;
    xml: TLaXmlFile;
  begin
    // Read data from an XML file
    xml := TLaXmlFile.Create;
    try
      xml.Load(fileName);

      for i := 0 to xml.Root.Count - 1 do
        Add(xml.Root[i].Value);
    finally
      xml.Free;
    end;
  end;

var
  header: THeader;

  procedure OpenText;
  var
    codePage: Integer;
    stream: TFileStream;

    function Eof: Boolean;
    begin
      Result := stream.Position >= stream.Size;
    end;

    function ReadAnsi: WideString;
    var
      b: Byte;
      str: AnsiString;
    begin
      // Read a text line from an Ansi file
      str := '';

      while not Eof do
      begin
        stream.Read(b, Sizeof(b));

        if (b = 10) or (b = 13) then
        begin
          // New line found. Skip this and next characters
          stream.Read(b, Sizeof(b));
          Break;
        end;

        str := str + AnsiChar(b);
      end;

      // Convert Ansi string to Unicode string
      Result := AnsiToUnicode(str, codePage);
    end;

    function ReadUtf8: WideString;
    var
      b: Byte;
      str: RawByteString;
    begin
      // Read a text line from a UTF-8 file
      str := '';

      while not Eof do
      begin
        stream.Read(b, Sizeof(b));

        if (b = 10) or (b = 13) then
        begin
          // New line found. Skip this and next characters
          stream.Read(b, Sizeof(b));
          Break;
        end;

        str := str + AnsiChar(b);
      end;

      // Convert UTF-8 string to Unicode string
      Result := Utf8ToString(str);
    end;

    function ReadUtf16(bigEndian: Boolean): WideString;
    var
      c: WideChar;
    begin
      // Read a text line from a UTF-16 file
      Result := '';

      while not Eof do
      begin
        stream.Read(c, Sizeof(c));

        if bigEndian then
          c := ChangeWideCharByteOrder(c);

        if (c = #10) or (c = #13) then
        begin
          // New line found. Skip this and next characters
          stream.Read(c, Sizeof(c));
          Break;
        end;

        Result := Result + c;
      end;
    end;

  begin
    // Read data from a text file
    FileFormat := ffText;

    stream := TFileStream.Create(fileName, fmOpenRead);
    try
      if (header[0] = $FF) and (header[1] = $FE) then
      begin
        // UTF-16LE file
        codePage := UTF16LE_C;
        stream.Seek(2, soCurrent);

        while not Eof do
          Add(ReadUtf16(False));
      end
      else if (header[0] = $FE) and (header[1] = $FF) then
      begin
        // UTF-16BE file
        codePage := UTF16BE_C;
        stream.Seek(2, soCurrent);

        while not Eof do
          Add(ReadUtf16(True));
      end
      else if (header[0] = UTF8_BOM_C[0]) and (header[1] = UTF8_BOM_C[1]) and (header[2] = UTF8_BOM_C[2]) then
      begin
        // UTF-8 file
        codePage := UTF8_C;
        stream.Seek(3, soCurrent);

        while not Eof do
          Add(ReadUtf8);
      end
      else
      begin
        // Code page file. read the code page from the firs line
        codePage := StrToInt(ReadAnsi);

        while not Eof do
          Add(ReadAnsi);
      end;
    finally
      stream.Free;
    end;

    Self.CodePage := codePage;
  end;

  procedure OpenBinary;
  var
    stream: TFileStream;

    function ReadWord: Word;
    begin
      stream.Read(Result, Sizeof(Result));
    end;

  var
    i, j, len, count: Integer;
    str: String;
    utf8: Utf8String;
    ansi: AnsiString;
  begin
    FileFormat := ffBinary;

    stream := TFileStream.Create(fileName, fmOpenRead);
    try
      stream.Read(header, Sizeof(header));
      CodePage := ReadWord;
      count := ReadWord;

      for i := 0 to Count - 1 do
      begin
        if CodePage = UTF8_C then
        begin
          len := ReadWord;
          SetLength(utf8, len);
          stream.Read(utf8[1], len);
          str := Utf8ToString(utf8);
        end
        else if CodePage = UTF16LE_C then
        begin
          len := ReadWord;
          SetLength(str, len);
          stream.Read(str[1], 2*len);
        end
        else if CodePage = UTF16BE_C then
        begin
          len := ReadWord;
          SetLength(str, len);
          stream.Read(str[1], 2*len);

          // Change byte order of the string
          for j := 1 to len do
            str[j] := ChangeWideCharByteOrder(str[j]);
        end
        else
        begin
          len := ReadWord;
          SetLength(ansi, len);
          stream.Read(ansi[1], len);
          str := AnsiToUnicode(ansi, CodePage);
        end;

        Add(str);
      end;
    finally
      stream.Free;
    end;
  end;

var
  stream: TFileStream;
begin
  Clear;
  FFileName := fileName;

  if TLaXmlFile.IsXmlFile(fileName) then
    OpenXml
  else
  begin
    stream := TFileStream.Create(fileName, fmOpenRead);
    try
      stream.Read(header, Sizeof(header));
    finally
      stream.Free;
    end;

    if HeadersEqual(header, HEADER_C) then
      OpenBinary
    else
      OpenText;
  end;

  UpdateComponents;
end;

procedure TForm1.Populate(one, two, three, four, five: String; codePage: Integer);
begin
  Clear;

  Add(one);
  Add(two);
  Add(three);
  Add(four);
  Add(five);

  if FileFormat = ffXml then
    Self.CodePage := UTF8_C
  else
    Self.CodePage := codePage;

  UpdateComponents;
end;

procedure TForm1.FormCreate(Sender: TObject);

  function EnumCodePages(value: PChar): Integer; stdcall;
  var
    codePage: Integer;
    info: TCPInfoEx;
  begin
    codePage := StrToInt(value);
    GetCPINfoEx(codePage, 0, info);
    enumCombo.Items.AddObject(info.CodePageName, TObject(codePage));

    Result := 1;
  end;

begin
  enumCombo := EncodingCombo;
  EnumSystemCodePages(@EnumCodePages, CP_INSTALLED);
  EncodingCombo.Sorted := True;

  EncodingCombo.Items.InsertObject(0, 'UTF-8', TObject(UTF8_C));
  EncodingCombo.Items.InsertObject(1, 'UTF-16LE', TObject(UTF16LE_C));
  EncodingCombo.Items.InsertObject(2, 'UTF-16BE', TObject(UTF16BE_C));

  FileFormat := ffXml;
  EnglishMenuClick(Self);
end;

procedure TForm1.NewMenuClick(Sender: TObject);
begin
  FFileName := '';
  Clear;
  CodePage := UTF8_C;
  UpdateComponents;
end;

procedure TForm1.OpenMenuClick(Sender: TObject);
resourcestring
  SOpen = 'Open';
  SAllFiles = 'All files';
  SAllSupportedFiles = 'All supported files';
var
  str: String;

  procedure AddFormat(const description, ext: String);
  begin
    str := str + Format('%0:s (*.%1:s)|*.%1:s|', [description, ext]);
  end;

var
  dialog: TOpenDialog;
begin
  dialog := TOpenDialog.Create(nil);
  try
    dialog.Title := SOpen;

    str := Format('%0:s (*.%1:s;*.%2:s;*.%3:s)|*.%1:s;*.%2:s;*.%3:s|', [SAllSupportedFiles, XML_C, TEXT_C, BINARY_C]);
    AddFormat(SXmlFiles, XML_C);
    AddFormat(STextFiles, TEXT_C);
    AddFormat(SBinaryFiles, BINARY_C);
    str := str + Format('%s|*.*|', [SAllFiles]);
    dialog.Filter := str;

    dialog.DefaultExt := TEXT_C;

    if dialog.Execute then
      Open(dialog.FileName);
  finally
    dialog.Free;
  end;
end;

procedure TForm1.SaveMenuClick(Sender: TObject);
begin
  if FFileName <> '' then
    Save(FFileName)
  else
    SaveAsMenuClick(Self);
end;

procedure TForm1.SaveAsMenuClick(Sender: TObject);
resourcestring
  SSaveAs = 'Save As';
var
  dialog: TSaveDialog;
begin
  dialog := TSaveDialog.Create(nil);
  try
    dialog.Title := SSaveAs;
    dialog.FileName := FFileName;
    dialog.InitialDir := ExtractFileDir(Application.ExeName);
    dialog.Options := dialog.Options + [ofOverwritePrompt];

    if FileFormat = ffXml  then
    begin
      dialog.Filter := Format('%0:s (*.%1:s)|*.%1:s|', [SXmlFiles, XML_C]);
      dialog.DefaultExt := XML_C;
    end
    else if FileFormat = ffText  then
    begin
      dialog.Filter := Format('%0:s (*.%1:s)|*.%1:s|', [STextFiles, TEXT_C]);
      dialog.DefaultExt := TEXT_C;
    end
    else
    begin
      dialog.Filter := Format('%s (*.%1:s)|*.%1:s|', [SBinaryFiles, BINARY_C]);
      dialog.DefaultExt := BINARY_C;
    end;

    if dialog.Execute then
      Save(dialog.FileName);
  finally
    dialog.Free;
  end;
end;

procedure TForm1.ExitMenuClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.EnglishMenuClick(Sender: TObject);
begin
  Populate('One', 'Two', 'Three', 'Four', 'Five', WESTERN_CP_C);
end;

procedure TForm1.FinnishFinnishClick(Sender: TObject);
begin
  Populate('Yksi', 'Kaksi', 'Kolme', 'Neljä', 'Viisi', WESTERN_CP_C);
end;

procedure TForm1.FrenchMenuClick(Sender: TObject);
begin
  Populate('Un', 'Deux', 'Trois', 'Quatre', 'Cinq', WESTERN_CP_C);
end;

procedure TForm1.GermanMenuClick(Sender: TObject);
begin
  Populate('Eins', 'Zwei', 'Drei', 'Vier', 'Fünf', WESTERN_CP_C);
end;

procedure TForm1.JapaneseMenuClick(Sender: TObject);
begin
  Populate('一', '二', '三', '四', '五', JAPANESE_CP_C);
end;

procedure TForm1.PolishMenuClick(Sender: TObject);
begin
  Populate('Jeden', 'Dwa', 'Trzy', 'Cztery', 'Pięć', EASTERN_CP_C);
end;

procedure TForm1.SwedishMenuClick(Sender: TObject);
begin
  Populate('Ett', 'Två', 'Tre', 'Fyra', 'Fem', WESTERN_CP_C);
end;

procedure TForm1.AboutMenuClick(Sender: TObject);
resourcestring
  SAbout = 'This application shows how to read and write text and binary files using different character encodings.';
begin
  MessageDlg(SAbout, mtInformation, [mbOk], 0);
end;

procedure TForm1.ItemsMemoChange(Sender: TObject);
begin
  UpdateComponents;
end;

end.
